'''OpenGL extension MESA.framebuffer_flip_x

This module customises the behaviour of the 
OpenGL.raw.GLES2.MESA.framebuffer_flip_x to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension defines a new framebuffer parameter,
	GL_FRAMEBUFFER_FLIP_X_MESA, that changes the behavior of the reads and
	writes to the framebuffer attachment points. When
	GL_FRAMEBUFFER_FLIP_X_MESA is GL_TRUE, render commands and pixel transfer
	operations access the backing store of each attachment point with an
	x-inverted coordinate system. This x-inversion is relative to the
	coordinate system set when GL_FRAMEBUFFER_FLIP_X_MESA is GL_FALSE.
	
	Access through TexSubImage2D and similar calls will notice the effect of
	the flip when they are not attached to framebuffer objects because
	GL_FRAMEBUFFER_FLIP_X_MESA is associated with the framebuffer object and
	not the attachment points.
	
	This extension is mainly for pre-rotation and recommended to use it with
	MESA_framebuffer_flip_y and MESA_framebuffer_swap_xy to have rotated
	result.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/MESA/framebuffer_flip_x.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.MESA.framebuffer_flip_x import *
from OpenGL.raw.GLES2.MESA.framebuffer_flip_x import _EXTENSION_NAME

def glInitFramebufferFlipXMESA():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES2 import _types as _cs
# End users want this...
from OpenGL.raw.GLES2._types import *
from OpenGL.raw.GLES2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES2_ARM_shader_core_properties'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES2,'GLES2_ARM_shader_core_properties',error_checker=_errors._error_checker)
GL_SHADER_CORE_ACTIVE_COUNT_ARM=_C('GL_SHADER_CORE_ACTIVE_COUNT_ARM',0x96F1)
GL_SHADER_CORE_COUNT_ARM=_C('GL_SHADER_CORE_COUNT_ARM',0x96F0)
GL_SHADER_CORE_FMA_RATE_ARM=_C('GL_SHADER_CORE_FMA_RATE_ARM',0x96F6)
GL_SHADER_CORE_MAX_WARP_COUNT_ARM=_C('GL_SHADER_CORE_MAX_WARP_COUNT_ARM',0x96F3)
GL_SHADER_CORE_PIXEL_RATE_ARM=_C('GL_SHADER_CORE_PIXEL_RATE_ARM',0x96F4)
GL_SHADER_CORE_PRESENT_MASK_ARM=_C('GL_SHADER_CORE_PRESENT_MASK_ARM',0x96F2)
GL_SHADER_CORE_TEXEL_RATE_ARM=_C('GL_SHADER_CORE_TEXEL_RATE_ARM',0x96F5)
@_f
@_p.types(None,_cs.GLuint)
def glMaxActiveShaderCoresARM(count):pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES2 import _types as _cs
# End users want this...
from OpenGL.raw.GLES2._types import *
from OpenGL.raw.GLES2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES2_EXT_fragment_shading_rate'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES2,'GLES2_EXT_fragment_shading_rate',error_checker=_errors._error_checker)
GL_FRAGMENT_SHADING_RATE_ATTACHMENT_WITH_DEFAULT_FRAMEBUFFER_SUPPORTED_EXT=_C('GL_FRAGMENT_SHADING_RATE_ATTACHMENT_WITH_DEFAULT_FRAMEBUFFER_SUPPORTED_EXT',0x96DF)
GL_FRAGMENT_SHADING_RATE_COMBINER_OP_KEEP_EXT=_C('GL_FRAGMENT_SHADING_RATE_COMBINER_OP_KEEP_EXT',0x96D2)
GL_FRAGMENT_SHADING_RATE_COMBINER_OP_MAX_EXT=_C('GL_FRAGMENT_SHADING_RATE_COMBINER_OP_MAX_EXT',0x96D5)
GL_FRAGMENT_SHADING_RATE_COMBINER_OP_MIN_EXT=_C('GL_FRAGMENT_SHADING_RATE_COMBINER_OP_MIN_EXT',0x96D4)
GL_FRAGMENT_SHADING_RATE_COMBINER_OP_MUL_EXT=_C('GL_FRAGMENT_SHADING_RATE_COMBINER_OP_MUL_EXT',0x96D6)
GL_FRAGMENT_SHADING_RATE_COMBINER_OP_REPLACE_EXT=_C('GL_FRAGMENT_SHADING_RATE_COMBINER_OP_REPLACE_EXT',0x96D3)
GL_FRAGMENT_SHADING_RATE_NON_TRIVIAL_COMBINERS_SUPPORTED_EXT=_C('GL_FRAGMENT_SHADING_RATE_NON_TRIVIAL_COMBINERS_SUPPORTED_EXT',0x8F6F)
GL_FRAGMENT_SHADING_RATE_WITH_SAMPLE_MASK_SUPPORTED_EXT=_C('GL_FRAGMENT_SHADING_RATE_WITH_SAMPLE_MASK_SUPPORTED_EXT',0x96DE)
GL_FRAGMENT_SHADING_RATE_WITH_SHADER_DEPTH_STENCIL_WRITES_SUPPORTED_EXT=_C('GL_FRAGMENT_SHADING_RATE_WITH_SHADER_DEPTH_STENCIL_WRITES_SUPPORTED_EXT',0x96DD)
GL_MAX_FRAGMENT_SHADING_RATE_ATTACHMENT_LAYERS_EXT=_C('GL_MAX_FRAGMENT_SHADING_RATE_ATTACHMENT_LAYERS_EXT',0x96DC)
GL_MAX_FRAGMENT_SHADING_RATE_ATTACHMENT_TEXEL_ASPECT_RATIO_EXT=_C('GL_MAX_FRAGMENT_SHADING_RATE_ATTACHMENT_TEXEL_ASPECT_RATIO_EXT',0x96DB)
GL_MAX_FRAGMENT_SHADING_RATE_ATTACHMENT_TEXEL_HEIGHT_EXT=_C('GL_MAX_FRAGMENT_SHADING_RATE_ATTACHMENT_TEXEL_HEIGHT_EXT',0x96DA)
GL_MAX_FRAGMENT_SHADING_RATE_ATTACHMENT_TEXEL_WIDTH_EXT=_C('GL_MAX_FRAGMENT_SHADING_RATE_ATTACHMENT_TEXEL_WIDTH_EXT',0x96D8)
GL_MIN_FRAGMENT_SHADING_RATE_ATTACHMENT_TEXEL_HEIGHT_EXT=_C('GL_MIN_FRAGMENT_SHADING_RATE_ATTACHMENT_TEXEL_HEIGHT_EXT',0x96D9)
GL_MIN_FRAGMENT_SHADING_RATE_ATTACHMENT_TEXEL_WIDTH_EXT=_C('GL_MIN_FRAGMENT_SHADING_RATE_ATTACHMENT_TEXEL_WIDTH_EXT',0x96D7)
GL_SHADING_RATE_1X1_PIXELS_EXT=_C('GL_SHADING_RATE_1X1_PIXELS_EXT',0x96A6)
GL_SHADING_RATE_1X2_PIXELS_EXT=_C('GL_SHADING_RATE_1X2_PIXELS_EXT',0x96A7)
GL_SHADING_RATE_1X4_PIXELS_EXT=_C('GL_SHADING_RATE_1X4_PIXELS_EXT',0x96AA)
GL_SHADING_RATE_2X1_PIXELS_EXT=_C('GL_SHADING_RATE_2X1_PIXELS_EXT',0x96A8)
GL_SHADING_RATE_2X2_PIXELS_EXT=_C('GL_SHADING_RATE_2X2_PIXELS_EXT',0x96A9)
GL_SHADING_RATE_2X4_PIXELS_EXT=_C('GL_SHADING_RATE_2X4_PIXELS_EXT',0x96AD)
GL_SHADING_RATE_4X1_PIXELS_EXT=_C('GL_SHADING_RATE_4X1_PIXELS_EXT',0x96AB)
GL_SHADING_RATE_4X2_PIXELS_EXT=_C('GL_SHADING_RATE_4X2_PIXELS_EXT',0x96AC)
GL_SHADING_RATE_4X4_PIXELS_EXT=_C('GL_SHADING_RATE_4X4_PIXELS_EXT',0x96AE)
GL_SHADING_RATE_ATTACHMENT_EXT=_C('GL_SHADING_RATE_ATTACHMENT_EXT',0x96D1)
GL_SHADING_RATE_EXT=_C('GL_SHADING_RATE_EXT',0x96D0)
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLuint,_cs.GLint,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei)
def glFramebufferShadingRateEXT(target,attachment,texture,baseLayer,numLayers,texelWidth,texelHeight):pass
@_f
@_p.types(None,_cs.GLsizei,_cs.GLsizei,arrays.GLsizeiArray,arrays.GLuintArray)
def glGetFragmentShadingRatesEXT(samples,maxCount,count,shadingRates):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum)
def glShadingRateCombinerOpsEXT(combinerOp0,combinerOp1):pass
@_f
@_p.types(None,_cs.GLenum)
def glShadingRateEXT(rate):pass



'''OpenGL extension MESA.bgra

This module customises the behaviour of the 
OpenGL.raw.GLES2.MESA.bgra to provide a more 
Python-friendly API

Overview (from the spec)
	
	MESA_bgra extends the list of combinations host-memory color formats
	with internal formats to include BGRA and BGR as acceptable formats
	with RGB8/SRGB8 and RGBA/sRGB8_ALPHA8 as internal formats respectively.
	This feature is of interest in virtualized environments, where the host
	supports OpenGL ES only, and the virtualized guest is supposed to support
	a subset of OpenGL including textures created with the format BGRA.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/MESA/bgra.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.MESA.bgra import *
from OpenGL.raw.GLES2.MESA.bgra import _EXTENSION_NAME

def glInitBgraMESA():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension OES.mapbuffer

This module customises the behaviour of the 
OpenGL.raw.GLES1.OES.mapbuffer to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds to the vertex buffer object functionality supported
	by OpenGL ES 1.1 or ES 2.0 by allowing the entire data storage of a
	buffer object to be mapped into the client's address space.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/OES/mapbuffer.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES1 import _types, _glgets
from OpenGL.raw.GLES1.OES.mapbuffer import *
from OpenGL.raw.GLES1.OES.mapbuffer import _EXTENSION_NAME

def glInitMapbufferOES():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

glGetBufferPointervOES=wrapper.wrapper(glGetBufferPointervOES).setInputArraySize(
    'params', 1
)
### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_OVR_multiview'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_OVR_multiview',error_checker=_errors._error_checker)
GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_BASE_VIEW_INDEX_OVR=_C('GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_BASE_VIEW_INDEX_OVR',0x9632)
GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_NUM_VIEWS_OVR=_C('GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_NUM_VIEWS_OVR',0x9630)
GL_FRAMEBUFFER_INCOMPLETE_VIEW_TARGETS_OVR=_C('GL_FRAMEBUFFER_INCOMPLETE_VIEW_TARGETS_OVR',0x9633)
GL_MAX_VIEWS_OVR=_C('GL_MAX_VIEWS_OVR',0x9631)
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLuint,_cs.GLint,_cs.GLint,_cs.GLsizei)
def glFramebufferTextureMultiviewOVR(target,attachment,texture,level,baseViewIndex,numViews):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLuint,_cs.GLint,_cs.GLint,_cs.GLsizei)
def glNamedFramebufferTextureMultiviewOVR(framebuffer,attachment,texture,level,baseViewIndex,numViews):pass



'''OpenGL extension MESA.sampler_objects

This module customises the behaviour of the 
OpenGL.raw.GLES2.MESA.sampler_objects to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension makes the sampler object subset of OpenGL ES 3.0 available
	in OpenGL ES 2.0 contexts. As the intent is to allow access to the API
	without necessarily requiring additional renderer functionality, some
	sampler state that would be mandatory in GLES 3 is dependent on the
	presence of additional extensions. Under GLES 3.0 or above this extension's
	name string may be exposed for compatibility, but it is otherwise without
	effect.
	
	Refer to the OpenGL ES 3.0 specification for API details not covered here.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/MESA/sampler_objects.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.MESA.sampler_objects import *
from OpenGL.raw.GLES2.MESA.sampler_objects import _EXTENSION_NAME

def glInitSamplerObjectsMESA():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

glGenSamplers=wrapper.wrapper(glGenSamplers).setOutput(
    'samplers',size=lambda x:(x,),pnameArg='count',orPassIn=True
)
# INPUT glDeleteSamplers.samplers size not checked against count
glDeleteSamplers=wrapper.wrapper(glDeleteSamplers).setInputArraySize(
    'samplers', None
)
# INPUT glSamplerParameteriv.param size not checked against 'pname'
glSamplerParameteriv=wrapper.wrapper(glSamplerParameteriv).setInputArraySize(
    'param', None
)
# INPUT glSamplerParameterfv.param size not checked against 'pname'
glSamplerParameterfv=wrapper.wrapper(glSamplerParameterfv).setInputArraySize(
    'param', None
)
glGetSamplerParameteriv=wrapper.wrapper(glGetSamplerParameteriv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetSamplerParameterfv=wrapper.wrapper(glGetSamplerParameterfv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
### END AUTOGENERATED SECTION


'''OpenGL extension EXT.EGL_image_storage_compression

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.EGL_image_storage_compression to provide a more 
Python-friendly API

Overview (from the spec)
	
	Applications may wish to take advantage of framebuffer compression. Some
	platforms may support framebuffer compression at fixed bitrates. Such
	compression algorithms generally produce results that are visually lossless,
	but the results are typically not bit-exact when compared to a non-compressed
	result.
	
	This extension enables applications to specify a texture by taking all
	properties from a fixed-rate compressed EGLImage.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/EGL_image_storage_compression.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.EGL_image_storage_compression import *
from OpenGL.raw.GLES2.EXT.EGL_image_storage_compression import _EXTENSION_NAME

def glInitEglImageStorageCompressionEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension NV.timeline_semaphore

This module customises the behaviour of the 
OpenGL.raw.GL.NV.timeline_semaphore to provide a more 
Python-friendly API

Overview (from the spec)
	
	The Vulkan API introduces the concept of timeline semaphores.
	This extension brings those concepts to the OpenGL API by adding
	a semaphore type to the semaphore object. In OpenGL, timeline semaphore
	signal and wait operations are similar to the corresponding operations on
	imported Direct3D 12 fences defined in EXT_external_objects_win32.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/timeline_semaphore.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NV.timeline_semaphore import *
from OpenGL.raw.GL.NV.timeline_semaphore import _EXTENSION_NAME

def glInitTimelineSemaphoreNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glCreateSemaphoresNV.semaphores size not checked against n
glCreateSemaphoresNV=wrapper.wrapper(glCreateSemaphoresNV).setInputArraySize(
    'semaphores', None
)
### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_AMD_framebuffer_sample_positions'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_AMD_framebuffer_sample_positions',error_checker=_errors._error_checker)
GL_ALL_PIXELS_AMD=_C('GL_ALL_PIXELS_AMD',0xFFFFFFFF)
GL_PIXELS_PER_SAMPLE_PATTERN_X_AMD=_C('GL_PIXELS_PER_SAMPLE_PATTERN_X_AMD',0x91AE)
GL_PIXELS_PER_SAMPLE_PATTERN_Y_AMD=_C('GL_PIXELS_PER_SAMPLE_PATTERN_Y_AMD',0x91AF)
GL_SUBSAMPLE_DISTANCE_AMD=_C('GL_SUBSAMPLE_DISTANCE_AMD',0x883F)
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,_cs.GLuint,arrays.GLfloatArray)
def glFramebufferSamplePositionsfvAMD(target,numsamples,pixelindex,values):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLuint,_cs.GLuint,_cs.GLsizei,arrays.GLfloatArray)
def glGetFramebufferParameterfvAMD(target,pname,numsamples,pixelindex,size,values):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLuint,_cs.GLuint,_cs.GLsizei,arrays.GLfloatArray)
def glGetNamedFramebufferParameterfvAMD(framebuffer,pname,numsamples,pixelindex,size,values):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLuint,arrays.GLfloatArray)
def glNamedFramebufferSamplePositionsfvAMD(framebuffer,numsamples,pixelindex,values):pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_ARB_draw_indirect'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_ARB_draw_indirect',error_checker=_errors._error_checker)
GL_DRAW_INDIRECT_BUFFER=_C('GL_DRAW_INDIRECT_BUFFER',0x8F3F)
GL_DRAW_INDIRECT_BUFFER_BINDING=_C('GL_DRAW_INDIRECT_BUFFER_BINDING',0x8F43)
@_f
@_p.types(None,_cs.GLenum,ctypes.c_void_p)
def glDrawArraysIndirect(mode,indirect):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,ctypes.c_void_p)
def glDrawElementsIndirect(mode,type,indirect):pass



'''OpenGL extension QCOM.frame_extrapolation

This module customises the behaviour of the 
OpenGL.raw.GLES2.QCOM.frame_extrapolation to provide a more 
Python-friendly API

Overview (from the spec)
	
	Frame extrapolation is the process of producing a new, future frame
	based on the contents of two previously rendered frames. It may be
	used to produce high frame rate display updates without incurring the
	full cost of traditional rendering at the higher framerate.
	
	This extension adds support for frame extrapolation in OpenGL ES by
	adding a function which takes three textures. The first two are used
	in sequence as the source frames, from which the extrapolated frame
	is derived. The extrapolated frame is stored in the third texture.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/QCOM/frame_extrapolation.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.QCOM.frame_extrapolation import *
from OpenGL.raw.GLES2.QCOM.frame_extrapolation import _EXTENSION_NAME

def glInitFrameExtrapolationQCOM():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension NV.instanced_arrays

This module customises the behaviour of the 
OpenGL.raw.GLES2.NV.instanced_arrays to provide a more 
Python-friendly API

Overview (from the spec)
	
	A common use case in GL for some applications is to be able to
	draw the same object, or groups of similar objects that share
	vertex data, primitive count and type, multiple times.  This
	extension provides a means of accelerating such use cases while
	limiting the number of required API calls, and keeping the amount
	of duplicate data to a minimum.
	
	In particular, this extension specifies an alternative to the
	read-only shader variable introduced by NV_draw_instanced.  It
	uses the same draw calls introduced by that extension, but
	redefines them so that a vertex shader can instead use vertex
	array attributes as a source of instance data.
	
	This extension introduces an array "divisor" for generic
	vertex array attributes, which when non-zero specifies that the
	attribute is "instanced."  An instanced attribute does not
	advance per-vertex as usual, but rather after every <divisor>
	conceptual draw calls.
	
	(Attributes which aren't instanced are repeated in their entirety
	for every conceptual draw call.)
	
	By specifying transform data in an instanced attribute or series
	of instanced attributes, vertex shaders can, in concert with the
	instancing draw calls, draw multiple instances of an object with
	one draw call.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/instanced_arrays.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.NV.instanced_arrays import *
from OpenGL.raw.GLES2.NV.instanced_arrays import _EXTENSION_NAME

def glInitInstancedArraysNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension QCOM.texture_foveated2

This module customises the behaviour of the 
OpenGL.raw.GLES2.QCOM.texture_foveated2 to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds a new texture parameter that specifies a pixel density
	below which rendering can be discarded during foveated rendering.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/QCOM/texture_foveated2.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.QCOM.texture_foveated2 import *
from OpenGL.raw.GLES2.QCOM.texture_foveated2 import _EXTENSION_NAME

def glInitTextureFoveated2QCOM():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES2 import _types as _cs
# End users want this...
from OpenGL.raw.GLES2._types import *
from OpenGL.raw.GLES2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES2_NV_conditional_render'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES2,'GLES2_NV_conditional_render',error_checker=_errors._error_checker)
GL_QUERY_BY_REGION_NO_WAIT_NV=_C('GL_QUERY_BY_REGION_NO_WAIT_NV',0x8E16)
GL_QUERY_BY_REGION_WAIT_NV=_C('GL_QUERY_BY_REGION_WAIT_NV',0x8E15)
GL_QUERY_NO_WAIT_NV=_C('GL_QUERY_NO_WAIT_NV',0x8E14)
GL_QUERY_WAIT_NV=_C('GL_QUERY_WAIT_NV',0x8E13)
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum)
def glBeginConditionalRenderNV(id,mode):pass
@_f
@_p.types(None,)
def glEndConditionalRenderNV():pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES2 import _types as _cs
# End users want this...
from OpenGL.raw.GLES2._types import *
from OpenGL.raw.GLES2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES2_EXT_framebuffer_blit_layers'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES2,'GLES2_EXT_framebuffer_blit_layers',error_checker=_errors._error_checker)

@_f
@_p.types(None,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLbitfield,_cs.GLenum)
def glBlitFramebufferLayerEXT(srcX0,srcY0,srcX1,srcY1,srcLayer,dstX0,dstY0,dstX1,dstY1,dstLayer,mask,filter):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLbitfield,_cs.GLenum)
def glBlitFramebufferLayersEXT(srcX0,srcY0,srcX1,srcY1,dstX0,dstY0,dstX1,dstY1,mask,filter):pass



"""Abstraction for the platform-specific code in PyOpenGL

Each supported platform has a module which provides the
specific functionality required to support the base OpenGL 
functionality on that platform.  These modules are 
registered using plugins in the:

    OpenGL.plugin.PlatformPlugin

objects.  To support a new platform you'll need to create
a new PlatformPlugin instance *before* you import 
OpenGL.platform .  Once you have a working platform 
module, please consider contributing it back to the project.

See baseplatform.BasePlatform for the core functionality 
of a platform implementation.  See the various platform 
specific modules for examples to use when porting.
"""

import os, sys
from OpenGL.plugins import PlatformPlugin
from OpenGL import _configflags

XDG = 'XDG_SESSION_TYPE'
WAYLAND_DISPLAY = 'WAYLAND_DISPLAY'

PLATFORM = None
nullFunction = None


def _load():
    """Load the os.name plugin for the platform functionality"""
    # Linux override keys...
    guessing_key = None
    if sys.platform in ('linux', 'linux2') and 'PYOPENGL_PLATFORM' not in os.environ:
        if 'WAYLAND_DISPLAY' in os.environ:
            guessing_key = 'wayland'
        elif 'DISPLAY' in os.environ:
            guessing_key = 'linux'

    key = (
        os.environ.get('PYOPENGL_PLATFORM'),
        os.environ.get('XDG_SESSION_TYPE', '').lower(),
        guessing_key,
        sys.platform,
        os.name,
    )
    plugin = PlatformPlugin.match(key)
    plugin_class = plugin.load()
    plugin.loaded = True
    # create instance of this platform implementation
    plugin = plugin_class()

    # install into the platform module's namespace now
    plugin.install(globals())
    return plugin


_load()


def types(resultType, *argTypes):
    """Decorator to add returnType, argTypes and argNames to a function"""

    def add_types(function):
        """Adds the given metadata to the function, introspects var names from declaration"""
        function.resultType = resultType
        function.argTypes = argTypes
        if hasattr(function, 'func_code'):  # python 2.x
            function.argNames = function.func_code.co_varnames
        else:
            function.argNames = function.__code__.co_varnames
        if _configflags.TYPE_ANNOTATIONS:
            function.__annotations__ = {
                'return': resultType,
            }
            for name, typ in zip(function.argNames, argTypes):
                function.__annotations__[name] = typ
        return function

    return add_types


def unpack_constants(constants, namespace):
    """Create constants and add to the namespace"""
    from OpenGL.constant import Constant

    for line in constants.splitlines():
        if line and line.split():
            name, value = line.split()
            namespace[name] = Constant(name, int(value, 16))


def createFunction(
    function,
    dll,
    extension,
    deprecated=False,
    error_checker=None,
    force_extension=False,
):
    """Allows the more compact declaration format to use the old-style constructor"""
    return nullFunction(
        function.__name__,
        dll or PLATFORM.GL,
        resultType=function.resultType,
        argTypes=function.argTypes,
        doc=None,
        argNames=function.argNames,
        extension=extension,
        deprecated=deprecated,
        module=function.__module__,
        error_checker=error_checker,
        force_extension=force_extension
        or getattr(function, 'force_extension', force_extension),
    )



#! /usr/bin/env python
"""Test for a buffer-protocol-based access mechanism

Will *only* work for Python 2.6+, and pretty much just works for strings
under 2.6 (in terms of the common object types).
"""
import sys, operator, logging, traceback
from OpenGL.arrays import _buffers
from OpenGL.raw.GL import _types

# from OpenGL.raw.GL.VERSION import GL_1_1
from OpenGL.arrays import formathandler
from OpenGL import _configflags
from OpenGL import acceleratesupport

_log = logging.getLogger(__name__)
try:
    reduce
except NameError as err:
    from functools import reduce

MemoryviewHandler = BufferHandler = None
if sys.version_info[:2] > (2, 6):
    # Only Python 2.7+ has memoryview support, and the accelerate module
    # requires memoryviews to be able to pass around the buffer structures
    if acceleratesupport.ACCELERATE_AVAILABLE:
        try:
            from OpenGL_accelerate.buffers_formathandler import MemoryviewHandler
        except ImportError as err:
            traceback.print_exc()
            _log.warning(
                "Unable to load buffers_formathandler accelerator from OpenGL_accelerate"
            )
        else:
            BufferHandler = MemoryviewHandler
if not BufferHandler:

    class BufferHandler(formathandler.FormatHandler):
        """Buffer-protocol data-type handler for OpenGL"""

        isOutput = False
        ERROR_ON_COPY = _configflags.ERROR_ON_COPY
        if sys.version_info[0] >= 3:

            @classmethod
            def from_param(cls, value, typeCode=None):
                if not isinstance(value, _buffers.Py_buffer):
                    value = cls.asArray(value)
                    # raise TypeError( """Can't convert value to py-buffer in from_param""" )
                # TODO: only do this IFF value.internal is None
                return _types.GLvoidp(value.buf)

        #                return value
        else:

            @classmethod
            def from_param(cls, value, typeCode=None):
                if not isinstance(value, _buffers.Py_buffer):
                    value = cls.asArray(value)
                    # raise TypeError( """Can't convert value to py-buffer in from_param""" )
                return value.buf

        def dataPointer(value):
            if not isinstance(value, _buffers.Py_buffer):
                value = _buffers.Py_buffer.from_object(value)
            return value.buf

        dataPointer = staticmethod(dataPointer)

        @classmethod
        def zeros(cls, dims, typeCode=None):
            """Currently don't allow strings as output types!"""
            raise NotImplementedError(
                "Generic buffer type does not have output capability"
            )
            return cls.asArray(
                bytearray(b'\000' * reduce(operator.mul, dims) * BYTE_SIZES[typeCode])
            )

        @classmethod
        def ones(cls, dims, typeCode=None):
            """Currently don't allow strings as output types!"""
            raise NotImplementedError("""Have not implemented ones for buffer type""")

        @classmethod
        def arrayToGLType(cls, value):
            """Given a value, guess OpenGL type of the corresponding pointer"""
            format = value.format
            if format in ARRAY_TO_GL_TYPE_MAPPING:
                return ARRAY_TO_GL_TYPE_MAPPING[format]
            raise TypeError('Unknown format: %r' % (format,))

        @classmethod
        def arraySize(cls, value, typeCode=None):
            """Given a data-value, calculate ravelled size for the array"""
            return value.len // value.itemsize

        @classmethod
        def arrayByteCount(cls, value, typeCode=None):
            """Given a data-value, calculate number of bytes required to represent"""
            return value.len

        @classmethod
        def unitSize(cls, value, default=None):
            return value.dims[-1]

        @classmethod
        def asArray(cls, value, typeCode=None):
            """Convert given value to an array value of given typeCode"""
            buf = _buffers.Py_buffer.from_object(value)
            return buf

        @classmethod
        def dimensions(cls, value, typeCode=None):
            """Determine dimensions of the passed array value (if possible)"""
            return value.dims


ARRAY_TO_GL_TYPE_MAPPING = _buffers.ARRAY_TO_GL_TYPE_MAPPING
BYTE_SIZES = _buffers.BYTE_SIZES



'''OpenGL extension OES.shader_image_atomic

This module customises the behaviour of the 
OpenGL.raw.GLES2.OES.shader_image_atomic to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides built-in functions allowing shaders to perform
	atomic read-modify-write operations to a single level of a texture
	object from any shader stage. These built-in functions are named
	imageAtomic*(), and accept integer texel coordinates to identify the
	texel accessed. These built-in functions extend the Images in ESSL 3.10.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/OES/shader_image_atomic.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.OES.shader_image_atomic import *
from OpenGL.raw.GLES2.OES.shader_image_atomic import _EXTENSION_NAME

def glInitShaderImageAtomicOES():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


"""Integer values looked up via glGetIntegerv( constant )"""

import ctypes

_get = None
_get_float = None


class LookupInt(object):
    def __init__(self, lookup, format=ctypes.c_int, calculation=None):
        self.lookup = lookup
        self.format = format
        self.calculation = calculation

    def __int__(self):
        global _get
        if _get is None:
            from OpenGL.GL import glGetIntegerv

            _get = glGetIntegerv
        output = self.format()
        _get(self.lookup, output)
        if self.calculation:
            return self.calculation(output.value)
        return output.value

    __long__ = __int__

    def __eq__(self, other):
        return int(self) == other

    def __cmp__(self, other):
        test = int(self)
        return (test > other) - (test < other)

    def __lt__(self, other):
        return int(self) < other

    def __gt__(self, other):
        return int(self) > other

    def __eq__(self, other):
        return int(self) == other

    def __str__(self):
        return str(int(self))

    __repr__ = __str__



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_EXT_texture_compression_rgtc'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_EXT_texture_compression_rgtc',error_checker=_errors._error_checker)
GL_COMPRESSED_RED_GREEN_RGTC2_EXT=_C('GL_COMPRESSED_RED_GREEN_RGTC2_EXT',0x8DBD)
GL_COMPRESSED_RED_RGTC1_EXT=_C('GL_COMPRESSED_RED_RGTC1_EXT',0x8DBB)
GL_COMPRESSED_SIGNED_RED_GREEN_RGTC2_EXT=_C('GL_COMPRESSED_SIGNED_RED_GREEN_RGTC2_EXT',0x8DBE)
GL_COMPRESSED_SIGNED_RED_RGTC1_EXT=_C('GL_COMPRESSED_SIGNED_RED_RGTC1_EXT',0x8DBC)




'''OpenGL extension QCOM.shading_rate

This module customises the behaviour of the 
OpenGL.raw.GLES2.QCOM.shading_rate to provide a more 
Python-friendly API

Overview (from the spec)
	
	By default, OpenGL runs a fragment shader once for each pixel covered by a
	primitive being rasterized.  When using multisampling, the outputs of that
	fragment shader are broadcast to each covered sample of the fragment's
	pixel.  When using multisampling, applications can optionally request that
	the fragment shader be run once per color sample (e.g., by using the "sample"
	qualifier on one or more active fragment shader inputs), or run a minimum
	number of times per pixel using SAMPLE_SHADING enable and the
	MinSampleShading frequency value.
	
	This extension allows applications to specify fragment shading rates of less
	than 1 invocation per pixel.  Instead of invoking the fragment shader
	once for each covered pixel, the fragment shader can be run once for a
	group of adjacent pixels in the framebuffer.  The outputs of that fragment
	shader invocation are broadcast to each covered samples for all of the pixels
	in the group.  The initial version of this extension allows for groups of
	1, 2, 4, 8, and 16 pixels.
	
	This can be useful for effects like motion volumetric rendering
	where a portion of scene is processed at full shading rate and a portion can
	be processed at a reduced shading rate, saving power and processing resources.
	The requested rate can vary from (finest and default) 1 fragment shader
	invocation per pixel to (coarsest) one fragment shader invocation for each
	4x4 block of pixels.  Implementations are given wide latitude to rasterize
	at the requested rate or any other rate that is less coarse.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/QCOM/shading_rate.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.QCOM.shading_rate import *
from OpenGL.raw.GLES2.QCOM.shading_rate import _EXTENSION_NAME

def glInitShadingRateQCOM():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES2 import _types as _cs
# End users want this...
from OpenGL.raw.GLES2._types import *
from OpenGL.raw.GLES2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES2_EXT_shader_framebuffer_fetch_non_coherent'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES2,'GLES2_EXT_shader_framebuffer_fetch_non_coherent',error_checker=_errors._error_checker)
GL_FRAGMENT_SHADER_DISCARDS_SAMPLES_EXT=_C('GL_FRAGMENT_SHADER_DISCARDS_SAMPLES_EXT',0x8A52)
@_f
@_p.types(None,)
def glFramebufferFetchBarrierEXT():pass



'''OpenGL extension MESA.texture_const_bandwidth

This module customises the behaviour of the 
OpenGL.raw.GL.MESA.texture_const_bandwidth to provide a more 
Python-friendly API

Overview (from the spec)
	
	The use of data dependent bandwidth compressed formats (UBWC, AFBC, etc)
	can introduce a form of side-channel, in that the bandwidth used for
	texture access is dependent on the texture's contents.  In some cases
	an application may want to disable the use of data dependent formats on
	specific textures.
	
	For that purpose, this extension extends EXT_memory_object to introduce
	a new <param> CONST_BW_TILING_MESA.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/MESA/texture_const_bandwidth.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.MESA.texture_const_bandwidth import *
from OpenGL.raw.GL.MESA.texture_const_bandwidth import _EXTENSION_NAME

def glInitTextureConstBandwidthMESA():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


"""Array data-type implementations (abstraction points for GL array types"""
import ctypes
import OpenGL

assert OpenGL
from OpenGL.raw.GL import _types
from OpenGL import plugins
from OpenGL.arrays import formathandler, _arrayconstants as GL_1_1
from OpenGL import logs

_log = logs.getLog("OpenGL.arrays.arraydatatype")
try:
    unicode
except NameError:
    unicode = str

from OpenGL import acceleratesupport

ADT = None
if acceleratesupport.ACCELERATE_AVAILABLE:
    try:
        from OpenGL_accelerate.arraydatatype import ArrayDatatype as ADT
    except ImportError as err:
        _log.warning("Unable to load ArrayDatatype accelerator from OpenGL_accelerate")
if ADT is None:
    # Python-coded version
    class HandlerRegistry(dict):
        GENERIC_OUTPUT_PREFERENCES = ["numpy", "ctypesarrays"]

        def __init__(self, plugin_match):
            self.match = plugin_match
            self.output_handler = None
            self.preferredOutput = None
            self.all_output_handlers = []

        def __call__(self, value):
            """Lookup of handler for given value"""
            try:
                typ = value.__class__
            except AttributeError:
                typ = type(value)
            handler = self.get(typ)
            if not handler:
                if hasattr(typ, "__mro__"):
                    for base in typ.__mro__:
                        handler = self.get(base)
                        if not handler:
                            handler = self.match(base)
                            if handler:
                                handler = handler.load()
                                if handler:
                                    handler = handler()
                        if handler:
                            self[typ] = handler
                            if hasattr(handler, "registerEquivalent"):
                                handler.registerEquivalent(typ, base)
                            return handler
                print(self.keys())
                raise TypeError(
                    """No array-type handler for type %s.%s (value: %s) registered"""
                    % (typ.__module__, typ.__name__, repr(value)[:50])
                )
            return handler

        def handler_by_plugin_name(self, name):
            plugin = plugins.FormatHandler.by_name(name)
            if plugin:
                try:
                    return plugin.load()
                except ImportError:
                    return None
            else:
                raise RuntimeError("No handler of name %s found" % (name,))

        def get_output_handler(self):
            """Fast-path lookup for output handler object"""
            if self.output_handler is None:
                if self.preferredOutput is not None:
                    self.output_handler = self.handler_by_plugin_name(
                        self.preferredOutput
                    )
                if not self.output_handler:
                    for preferred in self.GENERIC_OUTPUT_PREFERENCES:
                        self.output_handler = self.handler_by_plugin_name(preferred)
                        if self.output_handler:
                            break
                if not self.output_handler:
                    raise RuntimeError(
                        """Unable to find any output handler at all (not even ctypes/numpy ones!)"""
                    )
            return self.output_handler

        def register(self, handler, types=None):
            """Register this class as handler for given set of types"""
            if not isinstance(types, (list, tuple)):
                types = [types]
            for type in types:
                self[type] = handler
            if handler.isOutput:
                self.all_output_handlers.append(handler)

        def registerReturn(self, handler):
            """Register this handler as the default return-type handler"""
            if isinstance(handler, (str, unicode)):
                self.preferredOutput = handler
                self.output_handler = None
            else:
                self.preferredOutput = None
                self.output_handler = handler

    GLOBAL_REGISTRY = HandlerRegistry(plugins.FormatHandler.match)
    formathandler.FormatHandler.TYPE_REGISTRY = GLOBAL_REGISTRY

    class ArrayDatatype(object):
        """Mix-in for array datatype classes

        The ArrayDatatype marker essentially is used to mark a particular argument
        as having an "array" type, which means that it is eligible for handling
        via the arrays sub-package and its registered handlers.
        """

        typeConstant = None
        handler = GLOBAL_REGISTRY
        getHandler = GLOBAL_REGISTRY.__call__
        returnHandler = GLOBAL_REGISTRY.get_output_handler
        isAccelerated = False

        @classmethod
        def getRegistry(cls):
            """Get our handler registry"""
            return cls.handler

        def from_param(cls, value, typeConstant=None):
            """Given a value in a known data-pointer type, convert to a ctypes pointer"""
            return cls.getHandler(value).from_param(value, cls.typeConstant)

        from_param = classmethod(logs.logOnFail(from_param, _log))

        def dataPointer(cls, value):
            """Given a value in a known data-pointer type, return long for pointer"""
            try:
                return cls.getHandler(value).dataPointer(value)
            except Exception:
                _log.warning(
                    """Failure in dataPointer for %s instance %s""",
                    type(value),
                    value,
                )
                raise

        dataPointer = classmethod(logs.logOnFail(dataPointer, _log))

        def voidDataPointer(cls, value):
            """Given value in a known data-pointer type, return void_p for pointer"""
            pointer = cls.dataPointer(value)
            try:
                return ctypes.c_void_p(pointer)
            except TypeError:
                return pointer

        voidDataPointer = classmethod(logs.logOnFail(voidDataPointer, _log))

        def typedPointer(cls, value):
            """Return a pointer-to-base-type pointer for given value"""
            return ctypes.cast(cls.dataPointer(value), ctypes.POINTER(cls.baseType))

        typedPointer = classmethod(typedPointer)

        def asArray(cls, value, typeCode=None):
            """Given a value, convert to preferred array representation"""
            return cls.getHandler(value).asArray(value, typeCode or cls.typeConstant)

        asArray = classmethod(logs.logOnFail(asArray, _log))

        def arrayToGLType(cls, value):
            """Given a data-value, guess the OpenGL type of the corresponding pointer

            Note: this is not currently used in PyOpenGL and may be removed
            eventually.
            """
            return cls.getHandler(value).arrayToGLType(value)

        arrayToGLType = classmethod(logs.logOnFail(arrayToGLType, _log))

        def arraySize(cls, value, typeCode=None):
            """Given a data-value, calculate dimensions for the array (number-of-units)"""
            return cls.getHandler(value).arraySize(value, typeCode or cls.typeConstant)

        arraySize = classmethod(logs.logOnFail(arraySize, _log))

        def unitSize(cls, value, typeCode=None):
            """Determine unit size of an array (if possible)

            Uses our local type if defined, otherwise asks the handler to guess...
            """
            return cls.getHandler(value).unitSize(value, typeCode or cls.typeConstant)

        unitSize = classmethod(logs.logOnFail(unitSize, _log))

        def zeros(cls, dims, typeCode=None):
            """Allocate a return array of the given dimensions filled with zeros"""
            return cls.returnHandler().zeros(dims, typeCode or cls.typeConstant)

        zeros = classmethod(logs.logOnFail(zeros, _log))

        def dimensions(cls, value):
            """Given a data-value, get the dimensions (assumes full structure info)"""
            return cls.getHandler(value).dimensions(value)

        dimensions = classmethod(logs.logOnFail(dimensions, _log))

        def arrayByteCount(cls, value):
            """Given a data-value, try to determine number of bytes it's final form occupies

            For most data-types this is arraySize() * atomic-unit-size
            """
            return cls.getHandler(value).arrayByteCount(value)

        arrayByteCount = classmethod(logs.logOnFail(arrayByteCount, _log))

    # the final array data-type classes...
    class GLclampdArray(ArrayDatatype, ctypes.POINTER(_types.GLclampd)):
        """Array datatype for GLclampd types"""

        baseType = _types.GLclampd
        typeConstant = _types.GL_DOUBLE

    class GLclampfArray(ArrayDatatype, ctypes.POINTER(_types.GLclampf)):
        """Array datatype for GLclampf types"""

        baseType = _types.GLclampf
        typeConstant = _types.GL_FLOAT

    class GLfloat16Array(ArrayDatatype, ctypes.POINTER(_types.GLushort)):
        """Array datatype for float16 as GLushort types"""

        baseType = _types.GLushort
        typeConstant = _types.GL_HALF_FLOAT

    class GLfloatArray(ArrayDatatype, ctypes.POINTER(_types.GLfloat)):
        """Array datatype for GLfloat types"""

        baseType = _types.GLfloat
        typeConstant = _types.GL_FLOAT

    class GLdoubleArray(ArrayDatatype, ctypes.POINTER(_types.GLdouble)):
        """Array datatype for GLdouble types"""

        baseType = _types.GLdouble
        typeConstant = _types.GL_DOUBLE

    class GLbyteArray(ArrayDatatype, ctypes.POINTER(_types.GLbyte)):
        """Array datatype for GLbyte types"""

        baseType = _types.GLbyte
        typeConstant = _types.GL_BYTE

    class GLcharArray(ArrayDatatype, ctypes.c_char_p):
        """Array datatype for ARB extension pointers-to-arrays"""

        baseType = _types.GLchar
        typeConstant = _types.GL_BYTE

    GLcharARBArray = GLcharArray

    class GLshortArray(ArrayDatatype, ctypes.POINTER(_types.GLshort)):
        """Array datatype for GLshort types"""

        baseType = _types.GLshort
        typeConstant = _types.GL_SHORT

    class GLintArray(ArrayDatatype, ctypes.POINTER(_types.GLint)):
        """Array datatype for GLint types"""

        baseType = _types.GLint
        typeConstant = _types.GL_INT

    class GLubyteArray(ArrayDatatype, ctypes.POINTER(_types.GLubyte)):
        """Array datatype for GLubyte types"""

        baseType = _types.GLubyte
        typeConstant = _types.GL_UNSIGNED_BYTE

    GLbooleanArray = GLubyteArray

    class GLushortArray(ArrayDatatype, ctypes.POINTER(_types.GLushort)):
        """Array datatype for GLushort types"""

        baseType = _types.GLushort
        typeConstant = _types.GL_UNSIGNED_SHORT

    class GLuintArray(ArrayDatatype, ctypes.POINTER(_types.GLuint)):
        """Array datatype for GLuint types"""

        baseType = _types.GLuint
        typeConstant = _types.GL_UNSIGNED_INT

    class GLint64Array(ArrayDatatype, ctypes.POINTER(_types.GLint64)):
        """Array datatype for GLuint types"""

        baseType = _types.GLint64
        typeConstant = None  # TODO: find out what this should be!

    class GLuint64Array(ArrayDatatype, ctypes.POINTER(_types.GLuint64)):
        """Array datatype for GLuint types"""

        baseType = _types.GLuint64
        typeConstant = _types.GL_UNSIGNED_INT64

    class GLenumArray(ArrayDatatype, ctypes.POINTER(_types.GLenum)):
        """Array datatype for GLenum types"""

        baseType = _types.GLenum
        typeConstant = _types.GL_UNSIGNED_INT

    class GLsizeiArray(ArrayDatatype, ctypes.POINTER(_types.GLsizei)):
        """Array datatype for GLsizei types"""

        baseType = _types.GLsizei
        typeConstant = _types.GL_INT

    class GLvoidpArray(ArrayDatatype, ctypes.POINTER(_types.GLvoid)):
        """Array datatype for GLenum types"""

        baseType = _types.GLvoidp
        typeConstant = _types.GL_VOID_P

    class GLfixedArray(ArrayDatatype, ctypes.POINTER(_types.GLfixed)):
        baseType = _types.GLfixed
        typeConstant = _types.GL_FIXED

else:
    # Cython-coded array handler
    _log.debug("Using accelerated ArrayDatatype")
    ArrayDatatype = ADT(None, None)
    GLclampdArray = ADT(GL_1_1.GL_DOUBLE, _types.GLclampd)
    GLclampfArray = ADT(GL_1_1.GL_FLOAT, _types.GLclampf)
    GLdoubleArray = ADT(GL_1_1.GL_DOUBLE, _types.GLdouble)
    GLfloat16Array = ADT(GL_1_1.GL_HALF_FLOAT, _types.GLushort)
    GLfloatArray = ADT(GL_1_1.GL_FLOAT, _types.GLfloat)
    GLbyteArray = ADT(GL_1_1.GL_BYTE, _types.GLbyte)
    GLcharArray = GLcharARBArray = ADT(GL_1_1.GL_BYTE, _types.GLchar)
    GLshortArray = ADT(GL_1_1.GL_SHORT, _types.GLshort)
    GLintArray = ADT(GL_1_1.GL_INT, _types.GLint)
    GLubyteArray = GLbooleanArray = ADT(GL_1_1.GL_UNSIGNED_BYTE, _types.GLubyte)
    GLushortArray = ADT(GL_1_1.GL_UNSIGNED_SHORT, _types.GLushort)
    GLuintArray = ADT(GL_1_1.GL_UNSIGNED_INT, _types.GLuint)
    GLint64Array = ADT(None, _types.GLint64)
    GLuint64Array = ADT(GL_1_1.GL_UNSIGNED_INT64, _types.GLuint64)
    GLenumArray = ADT(GL_1_1.GL_UNSIGNED_INT, _types.GLenum)
    GLsizeiArray = ADT(GL_1_1.GL_INT, _types.GLsizei)
    GLvoidpArray = ADT(_types.GL_VOID_P, _types.GLvoidp)
    GLfixedArray = ADT(_types.GL_FIXED, _types.GLfixed)

EGLAttribArray = GLintArray


GL_CONSTANT_TO_ARRAY_TYPE = {
    GL_1_1.GL_HALF_FLOAT: GLfloat16Array,
    GL_1_1.GL_DOUBLE: GLclampdArray,
    GL_1_1.GL_FLOAT: GLclampfArray,
    GL_1_1.GL_FLOAT: GLfloatArray,
    GL_1_1.GL_DOUBLE: GLdoubleArray,
    GL_1_1.GL_BYTE: GLbyteArray,
    GL_1_1.GL_SHORT: GLshortArray,
    GL_1_1.GL_INT: GLintArray,
    GL_1_1.GL_UNSIGNED_BYTE: GLubyteArray,
    GL_1_1.GL_UNSIGNED_SHORT: GLushortArray,
    GL_1_1.GL_UNSIGNED_INT: GLuintArray,
    _types.GL_FIXED: GLfixedArray,
    # GL_1_1.GL_UNSIGNED_INT : GLenumArray,
}



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.EGL import _types as _cs
# End users want this...
from OpenGL.raw.EGL._types import *
from OpenGL.raw.EGL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'EGL_KHR_stream_producer_eglsurface'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.EGL,'EGL_KHR_stream_producer_eglsurface',error_checker=_errors._error_checker)
EGL_STREAM_BIT_KHR=_C('EGL_STREAM_BIT_KHR',0x0800)
@_f
@_p.types(_cs.EGLSurface,_cs.EGLDisplay,_cs.EGLConfig,_cs.EGLStreamKHR,arrays.GLintArray)
def eglCreateStreamProducerSurfaceKHR(dpy,config,stream,attrib_list):pass



'''OpenGL extension ARB.direct_state_access

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.direct_state_access to provide a more 
Python-friendly API

Overview (from the spec)
	
	In unextended OpenGL, most mutation of state contained in objects is through
	an indirection known as a binding. Objects are attached to a context (either
	directly or indirectly via a container) and then commands to modify or
	query their state are issued on that context, indirecting through its
	attachments and into the underlying object. This is known as `bind-to-edit'.
	
	This extension derives from the GL_EXT_direct_state_access extension, which
	added accessors for most state on most objects, allowing it to be queried
	and modified without the object needing to be bound to a context. In cases
	where a single property of an object is to be modified, directly accessing
	its state can be more efficient than binding the object to the context and
	then indirecting through it. Further, directly accessing the state of
	objects through their names rather than by bind-to-edit does not disturb
	the bindings of the current context, which is useful for tools, middleware
	and other applications that are unaware of the outer state but it can also
	avoid cases of redundant state changes.
	
	There are several subtle differences between this extension and the older
	GL_EXT_direct_state_access extension. First, this extension only expands
	functionality that still exists in core profile OpenGL. Second, any function
	that only partially avoids bind-to-edit (for example, explicitly specifying
	a texture unit, bypassing the active texture selector but still indirecting
	through a texture binding) has been omitted. Finally, the original extension
	effectively allowed any function to create new objects whereas in unextended
	OpenGL, only binding functions created objects (bind-to-create), even if
	their names were obtained through one of the glGen* functions. This
	extension does not allow on-the-spot creation of objects. Rather than rely
	on bind-to-create (which would defeat the purpose of the extension), we add
	glCreate* functions that produce new names that represent state vectors
	initialized to their default values. Due to this last change, several
	functions no longer require their <target> parameters, and so where
	applicable, this parameter is absent from this extension.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/direct_state_access.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.direct_state_access import *
from OpenGL.raw.GL.ARB.direct_state_access import _EXTENSION_NAME

def glInitDirectStateAccessARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glCreateTransformFeedbacks.ids size not checked against n
glCreateTransformFeedbacks=wrapper.wrapper(glCreateTransformFeedbacks).setInputArraySize(
    'ids', None
)
# INPUT glCreateBuffers.buffers size not checked against n
glCreateBuffers=wrapper.wrapper(glCreateBuffers).setInputArraySize(
    'buffers', None
)
# INPUT glNamedBufferStorage.data size not checked against size
glNamedBufferStorage=wrapper.wrapper(glNamedBufferStorage).setInputArraySize(
    'data', None
)
# INPUT glNamedBufferData.data size not checked against size
glNamedBufferData=wrapper.wrapper(glNamedBufferData).setInputArraySize(
    'data', None
)
# INPUT glNamedBufferSubData.data size not checked against size
glNamedBufferSubData=wrapper.wrapper(glNamedBufferSubData).setInputArraySize(
    'data', None
)
# INPUT glClearNamedBufferData.data size not checked against 'format,type'
glClearNamedBufferData=wrapper.wrapper(glClearNamedBufferData).setInputArraySize(
    'data', None
)
# INPUT glClearNamedBufferSubData.data size not checked against 'format,type'
glClearNamedBufferSubData=wrapper.wrapper(glClearNamedBufferSubData).setInputArraySize(
    'data', None
)
glGetNamedBufferPointerv=wrapper.wrapper(glGetNamedBufferPointerv).setInputArraySize(
    'params', 1
)
# INPUT glGetNamedBufferSubData.data size not checked against size
glGetNamedBufferSubData=wrapper.wrapper(glGetNamedBufferSubData).setInputArraySize(
    'data', None
)
# INPUT glCreateFramebuffers.framebuffers size not checked against n
glCreateFramebuffers=wrapper.wrapper(glCreateFramebuffers).setInputArraySize(
    'framebuffers', None
)
# INPUT glNamedFramebufferDrawBuffers.bufs size not checked against n
glNamedFramebufferDrawBuffers=wrapper.wrapper(glNamedFramebufferDrawBuffers).setInputArraySize(
    'bufs', None
)
# INPUT glInvalidateNamedFramebufferData.attachments size not checked against numAttachments
glInvalidateNamedFramebufferData=wrapper.wrapper(glInvalidateNamedFramebufferData).setInputArraySize(
    'attachments', None
)
# INPUT glInvalidateNamedFramebufferSubData.attachments size not checked against numAttachments
glInvalidateNamedFramebufferSubData=wrapper.wrapper(glInvalidateNamedFramebufferSubData).setInputArraySize(
    'attachments', None
)
# INPUT glClearNamedFramebufferiv.value size not checked against 'buffer'
glClearNamedFramebufferiv=wrapper.wrapper(glClearNamedFramebufferiv).setInputArraySize(
    'value', None
)
# INPUT glClearNamedFramebufferuiv.value size not checked against 'buffer'
glClearNamedFramebufferuiv=wrapper.wrapper(glClearNamedFramebufferuiv).setInputArraySize(
    'value', None
)
# INPUT glClearNamedFramebufferfv.value size not checked against 'buffer'
glClearNamedFramebufferfv=wrapper.wrapper(glClearNamedFramebufferfv).setInputArraySize(
    'value', None
)
# INPUT glCreateRenderbuffers.renderbuffers size not checked against n
glCreateRenderbuffers=wrapper.wrapper(glCreateRenderbuffers).setInputArraySize(
    'renderbuffers', None
)
# INPUT glCreateTextures.textures size not checked against n
glCreateTextures=wrapper.wrapper(glCreateTextures).setInputArraySize(
    'textures', None
)
# INPUT glCompressedTextureSubImage1D.data size not checked against imageSize
glCompressedTextureSubImage1D=wrapper.wrapper(glCompressedTextureSubImage1D).setInputArraySize(
    'data', None
)
# INPUT glCompressedTextureSubImage2D.data size not checked against imageSize
glCompressedTextureSubImage2D=wrapper.wrapper(glCompressedTextureSubImage2D).setInputArraySize(
    'data', None
)
# INPUT glCompressedTextureSubImage3D.data size not checked against imageSize
glCompressedTextureSubImage3D=wrapper.wrapper(glCompressedTextureSubImage3D).setInputArraySize(
    'data', None
)
# INPUT glTextureParameterfv.param size not checked against 'pname'
glTextureParameterfv=wrapper.wrapper(glTextureParameterfv).setInputArraySize(
    'param', None
)
# INPUT glTextureParameterIiv.params size not checked against 'pname'
glTextureParameterIiv=wrapper.wrapper(glTextureParameterIiv).setInputArraySize(
    'params', None
)
# INPUT glTextureParameterIuiv.params size not checked against 'pname'
glTextureParameterIuiv=wrapper.wrapper(glTextureParameterIuiv).setInputArraySize(
    'params', None
)
# INPUT glTextureParameteriv.param size not checked against 'pname'
glTextureParameteriv=wrapper.wrapper(glTextureParameteriv).setInputArraySize(
    'param', None
)
# INPUT glGetTextureImage.pixels size not checked against bufSize
glGetTextureImage=wrapper.wrapper(glGetTextureImage).setInputArraySize(
    'pixels', None
)
# INPUT glGetCompressedTextureImage.pixels size not checked against bufSize
glGetCompressedTextureImage=wrapper.wrapper(glGetCompressedTextureImage).setInputArraySize(
    'pixels', None
)
# INPUT glCreateVertexArrays.arrays size not checked against n
glCreateVertexArrays=wrapper.wrapper(glCreateVertexArrays).setInputArraySize(
    'arrays', None
)
# INPUT glVertexArrayVertexBuffers.buffers size not checked against count
# INPUT glVertexArrayVertexBuffers.offsets size not checked against count
# INPUT glVertexArrayVertexBuffers.strides size not checked against count
glVertexArrayVertexBuffers=wrapper.wrapper(glVertexArrayVertexBuffers).setInputArraySize(
    'buffers', None
).setInputArraySize(
    'offsets', None
).setInputArraySize(
    'strides', None
)
# INPUT glCreateSamplers.samplers size not checked against n
glCreateSamplers=wrapper.wrapper(glCreateSamplers).setInputArraySize(
    'samplers', None
)
# INPUT glCreateProgramPipelines.pipelines size not checked against n
glCreateProgramPipelines=wrapper.wrapper(glCreateProgramPipelines).setInputArraySize(
    'pipelines', None
)
# INPUT glCreateQueries.ids size not checked against n
glCreateQueries=wrapper.wrapper(glCreateQueries).setInputArraySize(
    'ids', None
)
### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_NV_timeline_semaphore'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_NV_timeline_semaphore',error_checker=_errors._error_checker)
GL_MAX_TIMELINE_SEMAPHORE_VALUE_DIFFERENCE_NV=_C('GL_MAX_TIMELINE_SEMAPHORE_VALUE_DIFFERENCE_NV',0x95B6)
GL_SEMAPHORE_TYPE_BINARY_NV=_C('GL_SEMAPHORE_TYPE_BINARY_NV',0x95B4)
GL_SEMAPHORE_TYPE_NV=_C('GL_SEMAPHORE_TYPE_NV',0x95B3)
GL_SEMAPHORE_TYPE_TIMELINE_NV=_C('GL_SEMAPHORE_TYPE_TIMELINE_NV',0x95B5)
GL_TIMELINE_SEMAPHORE_VALUE_NV=_C('GL_TIMELINE_SEMAPHORE_VALUE_NV',0x9595)
@_f
@_p.types(None,_cs.GLsizei,arrays.GLuintArray)
def glCreateSemaphoresNV(n,semaphores):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,arrays.GLintArray)
def glGetSemaphoreParameterivNV(semaphore,pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,arrays.GLintArray)
def glSemaphoreParameterivNV(semaphore,pname,params):pass



"""Extension module support methods

This module provides the tools required to check whether
an extension is available
"""

from OpenGL.latebind import LateBind
from OpenGL._bytes import bytes, unicode, as_8_bit
import OpenGL as root
import sys
import logging
import re

_log = logging.getLogger('OpenGL.extensions')
VERSION_PREFIX = as_8_bit('GL_VERSION_GL_')
CURRENT_GL_VERSION = None
AVAILABLE_GL_EXTENSIONS = []
AVAILABLE_GLU_EXTENSIONS = []

# version tuple -> list of implicitly included extensions...
VERSION_EXTENSIONS = [
    (
        (3, 0),
        [
            as_8_bit('GL_ARB_vertex_array_object'),
            as_8_bit('GL_ARB_texture_buffer_object'),
            as_8_bit('GL_ARB_framebuffer_object'),
            as_8_bit('GL_ARB_map_buffer_range'),
        ],
    ),
    (
        (3, 1),
        [
            as_8_bit('GL_ARB_copy_buffer'),
            as_8_bit('GL_ARB_uniform_buffer_object'),
        ],
    ),
    (
        (3, 2),
        [
            as_8_bit('GL_ARB_draw_elements_base_vertex'),
            as_8_bit('GL_ARB_provoking_vertex'),
            as_8_bit('GL_ARB_sync'),
            as_8_bit('GL_ARB_texture_multisample'),
        ],
    ),
    (
        (3, 3),
        [
            as_8_bit('GL_ARB_texture_multisample'),
            as_8_bit('GL_ARB_blend_func_extended'),
            as_8_bit('GL_ARB_sampler_objects'),
            as_8_bit('GL_ARB_explicit_attrib_location'),
            as_8_bit('GL_ARB_occlusion_query2'),
            as_8_bit('GL_ARB_shader_bit_encoding'),
            as_8_bit('GL_ARB_texture_rgb10_a2ui'),
            as_8_bit('GL_ARB_texture_swizzle'),
            as_8_bit('GL_ARB_timer_query'),
            as_8_bit('GL_ARB_vertex_type_2_10_10_10_rev'),
        ],
    ),
    (
        (4, 0),
        [
            as_8_bit('GL_ARB_texture_query_lod'),
            as_8_bit('GL_ARB_draw_indirect'),
            as_8_bit('GL_ARB_gpu_shader5'),
            as_8_bit('GL_ARB_gpu_shader_fp64'),
            as_8_bit('GL_ARB_shader_subroutine'),
            as_8_bit('GL_ARB_tessellation_shader'),
            as_8_bit('GL_ARB_texture_buffer_object_rgb32'),
            as_8_bit('GL_ARB_texture_cube_map_array'),
            as_8_bit('GL_ARB_texture_gather'),
            as_8_bit('GL_ARB_transform_feedback2'),
            as_8_bit('GL_ARB_transform_feedback3'),
        ],
    ),
    (
        (4, 1),
        [
            as_8_bit('GL_ARB_ES2_compatibility'),
            as_8_bit('GL_ARB_get_program_binary'),
            as_8_bit('GL_ARB_separate_shader_objects'),
            as_8_bit('GL_ARB_shader_precision'),
            as_8_bit('GL_ARB_vertex_attrib_64bit'),
            as_8_bit('GL_ARB_viewport_array'),
        ],
    ),
    (
        (4, 2),
        [
            as_8_bit('GL_ARB_base_instance'),
            as_8_bit('GL_ARB_shading_language_420pack'),
            as_8_bit('GL_ARB_transform_feedback_instanced'),
            as_8_bit('GL_ARB_compressed_texture_pixel_storage'),
            as_8_bit('GL_ARB_conservative_depth'),
            as_8_bit('GL_ARB_internalformat_query'),
            as_8_bit('GL_ARB_map_buffer_alignment'),
            as_8_bit('GL_ARB_shader_atomic_counters'),
            as_8_bit('GL_ARB_shader_image_load_store'),
            as_8_bit('GL_ARB_shading_language_packing'),
            as_8_bit('GL_ARB_texture_storage'),
        ],
    ),
]


class ExtensionQuerier(object):
    prefix = None
    version_prefix = None
    assumed_version = [1, 0]

    version = extensions = None
    version_string = extensions_string = None

    registered = []

    def __init__(self):
        self.registered.append(self)

    @classmethod
    def hasExtension(self, specifier):
        for registered in self.registered:
            result = registered(specifier)
            if result:
                return result
        return False

    def __call__(self, specifier):
        specifier = as_8_bit(specifier).replace(as_8_bit('.'), as_8_bit('_'))
        if not specifier.startswith(as_8_bit(self.prefix)):
            return None

        if specifier.startswith(as_8_bit(self.version_prefix)):
            specifier = [
                int(x)
                for x in specifier[len(self.version_prefix) :].split(as_8_bit('_'))
            ]
            if specifier[:2] <= self.assumed_version:
                return True
            version = self.getVersion()
            if not version:
                return version
            return specifier <= version
        else:
            extensions = self.getExtensions()
            return extensions and specifier in extensions

    def getVersion(self):
        if not self.version:
            self.version = self.pullVersion()
        return self.version

    def getExtensions(self):
        if not self.extensions:
            self.extensions = self.pullExtensions()
        return self.extensions


class _GLQuerier(ExtensionQuerier):
    prefix = as_8_bit('GL_')
    version_prefix = as_8_bit('GL_VERSION_GL_')
    assumed_version = [1, 1]

    # If you call OpenGL's glGetString for a context which is
    # an ES context, you get an ES-style string, here we're going
    # to parse out the ES value, but this is one of those "the issue is the usage"
    # problems. You shouldn't do this, but if you do, this will make
    # the issue clearer
    version_matcher = re.compile(
        r'^(?P<api_marker>OpenGL (ES )?)?(?P<version>\d+([.]\d+)+)'
    )
    is_opengl_es = False

    def pullVersion(self):
        """Retrieve 2-int declaration of major/minor GL version

        returns [int(major),int(minor)] or False if not loaded
        """
        from OpenGL import platform

        if not platform.PLATFORM.CurrentContextIsValid():
            return False
        from OpenGL.raw.GL.VERSION.GL_1_1 import glGetString
        from OpenGL.raw.GL.VERSION.GL_1_1 import GL_VERSION

        new = glGetString(GL_VERSION)

        self.version_string = new.decode('utf-8', errors='ignore')
        match = self.version_matcher.match(self.version_string)
        if match:
            self.version_string = match.group('version')
            api_marker = match.group('api_marker')
            if api_marker:
                if api_marker == 'OpenGL ES':
                    _log.warning(
                        "Importing OpenGL in an OpenGL ES context is not advisable, generally"
                    )
                    self.is_opengl_es = True
        if new:
            return [int(x) for x in self.version_string.split('.')]
        else:
            return False  # not yet loaded/supported

    def pullExtensions(self):
        from OpenGL import platform

        if not platform.PLATFORM.CurrentContextIsValid():
            return False
        from OpenGL.raw.GL._types import GLint
        from OpenGL.raw.GL.VERSION.GL_1_1 import glGetString, glGetError
        from OpenGL.raw.GL.VERSION.GL_1_1 import GL_EXTENSIONS
        from OpenGL import error

        try:
            extensions = glGetString(GL_EXTENSIONS)
            if glGetError():
                raise error.GLError()
            if extensions:
                extensions = extensions.split()
            else:
                return False
        except (AttributeError, error.GLError):
            # OpenGL 3.0 deprecates glGetString( GL_EXTENSIONS )
            from OpenGL.raw.GL.VERSION.GL_3_0 import GL_NUM_EXTENSIONS, glGetStringi
            from OpenGL.raw.GL.VERSION.GL_1_1 import glGetIntegerv

            count = GLint()
            glGetIntegerv(GL_NUM_EXTENSIONS, count)
            extensions = []
            for i in range(count.value):
                extension = glGetStringi(GL_EXTENSIONS, i)
                extensions.append(extension)
        # Add included-by-reference extensions...
        version = self.getVersion()
        if not version:
            # should not be possible?
            return version
        check = tuple(version[:2])
        for v, v_exts in VERSION_EXTENSIONS:
            if v <= check:
                for v_ext in v_exts:
                    if v_ext not in extensions:
                        extensions.append(as_8_bit(v_ext))
            else:
                break
        return extensions


GLQuerier = _GLQuerier()


class _GLUQuerier(ExtensionQuerier):
    prefix = as_8_bit('GLU_')
    version_prefix = as_8_bit('GLU_VERSION_GL_')

    def pullVersion(self):
        from OpenGL.GLU import gluGetString, GLU_VERSION

        return [
            int(x)
            for x in gluGetString(GLU_VERSION).split(as_8_bit('_'))
            if x.isdigit()
        ]

    def pullExtensions(self):
        from OpenGL.GLU import gluGetString, GLU_EXTENSIONS

        return gluGetString(GLU_EXTENSIONS).split()


GLUQuerier = _GLUQuerier()


def hasExtension(specifier):
    return ExtensionQuerier.hasExtension(specifier)


hasGLExtension = hasGLUExtension = hasExtension


class _Alternate(LateBind):
    def __init__(self, name, *alternates):
        """Initialize set of alternative implementations of the same function"""
        self.__name__ = name
        self._alternatives = alternates
        if root.MODULE_ANNOTATIONS:
            frame = sys._getframe().f_back
            if frame and frame.f_back and '__name__' in frame.f_back.f_globals:
                self.__module__ = frame.f_back.f_globals['__name__']

    def __bool__(self):
        from OpenGL import error

        try:
            return bool(self.getFinalCall())
        except error.NullFunctionError as err:
            return False

    __nonzero__ = __bool__  # Python 2.6 compatibility

    def finalise(self):
        """Call, doing a late lookup and bind to find an implementation"""
        for alternate in self._alternatives:
            if alternate:
                #                _log.info(
                #                    """Chose alternate: %s from %s""",
                #                    alternate.__name__,
                #                    ", ".join([x.__name__ for x in self._alternatives])
                #                )
                return alternate
        from OpenGL import error

        raise error.NullFunctionError(
            """Attempt to call an undefined alternate function (%s), check for bool(%s) before calling"""
            % (
                ', '.join([x.__name__ for x in self._alternatives]),
                self.__name__,
            )
        )


def alternate(name, *functions):
    """Construct a callable that functions as the first implementation found of given set of alternatives

    if name is a function then its name will be used....
    """
    if not isinstance(name, (bytes, unicode)):
        functions = (name,) + functions
        name = name.__name__
    return type(name, (_Alternate,), {})(name, *functions)



'''OpenGL extension SGIX.ir_instrument1

This module customises the behaviour of the 
OpenGL.raw.GL.SGIX.ir_instrument1 to provide a more 
Python-friendly API

Overview (from the spec)


The official definition of this extension is available here:
http://www.opengl.org/registry/specs/SGIX/ir_instrument1.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.SGIX.ir_instrument1 import *
from OpenGL.raw.GL.SGIX.ir_instrument1 import _EXTENSION_NAME

def glInitIrInstrument1SGIX():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


"""EGL (cross-platform) platform library"""
import ctypes, ctypes.util
from OpenGL.platform import baseplatform, ctypesloader

class EGLPlatform( baseplatform.BasePlatform ):
    """EGL platform for opengl-es only platforms"""
    @baseplatform.lazy_property
    def GLES1(self):
        try:
            return ctypesloader.loadLibrary(
                ctypes.cdll,
                'GLESv1_CM', # ick
                mode=ctypes.RTLD_GLOBAL 
            )
        except OSError:
            return None
    @baseplatform.lazy_property
    def GLES2(self):
        try:
            return ctypesloader.loadLibrary(
                ctypes.cdll,
                'GLESv2', 
                mode=ctypes.RTLD_GLOBAL 
            )
        except OSError:
            return None
    @baseplatform.lazy_property
    def GLES3(self):
        # implementers guide says to use the same name for the DLL
        return self.GLES2
    @baseplatform.lazy_property
    def GL(self):
        try:
            for name in ('OpenGL','GL'):
                lib = ctypesloader.loadLibrary(
                    ctypes.cdll,
                    name, 
                    mode=ctypes.RTLD_GLOBAL 
                )
                if lib:
                    return lib 
            raise OSError("No GL/OpenGL library available")
        except OSError:
            return self.GLES2 or self.GLES1
    @baseplatform.lazy_property
    def GLU(self):
        try:
            return ctypesloader.loadLibrary(
                ctypes.cdll,
                'GLU',
                mode=ctypes.RTLD_GLOBAL 
            )
        except OSError:
            return None
    @baseplatform.lazy_property
    def GLUT( self ):
        try:
            return ctypesloader.loadLibrary(
                ctypes.cdll,
                'glut', 
                mode=ctypes.RTLD_GLOBAL 
            )
        except OSError:
            return None
    @baseplatform.lazy_property
    def OpenGL(self): return self.GL
    
    @baseplatform.lazy_property
    def EGL(self):
        # TODO: the raspberry pi crashes on trying to load EGL module 
        # because the EGL library requires a structure from GLES2 without 
        # linking to that library... Github issue is here:
        #   https://github.com/raspberrypi/firmware/issues/110
        import os
        if os.path.exists('/proc/cpuinfo'):
            with open('/proc/cpuinfo', 'r') as f:
                info = f.read()
            if 'BCM2708' in info or 'BCM2709' in info:
                assert self.GLES2
        try:
            return ctypesloader.loadLibrary(
                ctypes.cdll,
                'EGL', 
                mode=ctypes.RTLD_GLOBAL 
            )
        except OSError as err:
            raise ImportError("Unable to load EGL library", *err.args)
    @baseplatform.lazy_property
    def getExtensionProcedure( self ):
        eglGetProcAddress = self.EGL.eglGetProcAddress
        eglGetProcAddress.restype = ctypes.c_void_p
        return eglGetProcAddress
    @baseplatform.lazy_property
    def GLE( self ):
        try:
            return ctypesloader.loadLibrary(
                ctypes.cdll,
                'gle', 
                mode=ctypes.RTLD_GLOBAL 
            )
        except OSError:
            return None

    DEFAULT_FUNCTION_TYPE = staticmethod( ctypes.CFUNCTYPE )
    @baseplatform.lazy_property
    def GetCurrentContext( self ):
        eglGetCurrentContext = self.EGL.eglGetCurrentContext
        eglGetCurrentContext.restype = ctypes.c_void_p
        return eglGetCurrentContext

    def getGLUTFontPointer( self, constant ):
        """Platform specific function to retrieve a GLUT font pointer

        GLUTAPI void *glutBitmap9By15;
        #define GLUT_BITMAP_9_BY_15		(&glutBitmap9By15)

        Key here is that we want the addressof the pointer in the DLL,
        not the pointer in the DLL.  That is, our pointer is to the
        pointer defined in the DLL, we don't want the *value* stored in
        that pointer.
        """
        name = [ x.title() for x in constant.split( '_' )[1:] ]
        internal = 'glut' + "".join( [x.title() for x in name] )
        pointer = ctypes.c_void_p.in_dll( self.GLUT, internal )
        return ctypes.c_void_p(ctypes.addressof(pointer))

    def install(self, namespace):
        """Work around SDL not recognising wayland as platform by default"""
        result = super(EGLPlatform,self).install(namespace)
        import os
        if os.environ.get('XDG_SESSION_TYPE') == 'wayland':
            if not os.environ.get('SDL_VIDEODRIVER'):
                os.environ['SDL_VIDEODRIVER'] = 'wayland'
        return result


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLX import _types as _cs
# End users want this...
from OpenGL.raw.GLX._types import *
from OpenGL.raw.GLX import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLX_NV_multigpu_context'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLX,'GLX_NV_multigpu_context',error_checker=_errors._error_checker)
GLX_CONTEXT_MULTIGPU_ATTRIB_AFR_NV=_C('GLX_CONTEXT_MULTIGPU_ATTRIB_AFR_NV',0x20AC)
GLX_CONTEXT_MULTIGPU_ATTRIB_MULTICAST_NV=_C('GLX_CONTEXT_MULTIGPU_ATTRIB_MULTICAST_NV',0x20AD)
GLX_CONTEXT_MULTIGPU_ATTRIB_MULTI_DISPLAY_MULTICAST_NV=_C('GLX_CONTEXT_MULTIGPU_ATTRIB_MULTI_DISPLAY_MULTICAST_NV',0x20AE)
GLX_CONTEXT_MULTIGPU_ATTRIB_NV=_C('GLX_CONTEXT_MULTIGPU_ATTRIB_NV',0x20AA)
GLX_CONTEXT_MULTIGPU_ATTRIB_SINGLE_NV=_C('GLX_CONTEXT_MULTIGPU_ATTRIB_SINGLE_NV',0x20AB)




'''OpenGL extension ARM.texture_unnormalized_coordinates

This module customises the behaviour of the 
OpenGL.raw.GLES2.ARM.texture_unnormalized_coordinates to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides the option to switch to unnormalized
	coordinates for texture lookups using a sampler parameter.
	
	Texture lookup in OpenGL ES is done using normalized coordinates. For
	certain applications it is convenient to work with non-normalized
	coordinates instead. It also beneficial to keep support for bilinear
	filtering.
	
	Additional restrictions apply to textures with non-normalized
	coordinates that affect texture completeness and the available
	texture lookup functions.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARM/texture_unnormalized_coordinates.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.ARM.texture_unnormalized_coordinates import *
from OpenGL.raw.GLES2.ARM.texture_unnormalized_coordinates import _EXTENSION_NAME

def glInitTextureUnnormalizedCoordinatesARM():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension QCOM.render_sRGB_R8_RG8

This module customises the behaviour of the 
OpenGL.raw.GLES2.QCOM.render_sRGB_R8_RG8 to provide a more 
Python-friendly API

Overview (from the spec)
	
	OpenGL ES 3.2 supports an sRGB formats SR8 and SRG8 which are not color-renderable.
	
	This extension enables the sRGB formats SR8 and SRG8 to be color-renderable using
	framebuffer objects.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/QCOM/render_sRGB_R8_RG8.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.QCOM.render_sRGB_R8_RG8 import *
from OpenGL.raw.GLES2.QCOM.render_sRGB_R8_RG8 import _EXTENSION_NAME

def glInitRenderSrgbR8Rg8QCOM():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension EXT.platform_base

This module customises the behaviour of the 
OpenGL.raw.EGL.EXT.platform_base to provide a more 
Python-friendly API

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/platform_base.txt
'''
from OpenGL import constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.EGL import _types, _glgets
from OpenGL.raw.EGL.EXT.platform_base import *
from OpenGL.raw.EGL.EXT.platform_base import _EXTENSION_NAME

# Cannot use this to check for the extension because the extension
# checking requires a context
#eglGetPlatformDisplayEXT.extension = None

def glInitPlatformBaseEXT():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION
eglGetPlatformDisplayEXT.extension = None
eglGetPlatformDisplayEXT.force_extension = True 



"""Base class for the various Python data-format storage type APIs

Data-type handlers are specified using OpenGL.plugins module
"""

import ctypes
from OpenGL import plugins


class FormatHandler(object):
    """Abstract class describing the handler interface

    Each data-type handler is responsible for providing a number of methods
    which allow it to manipulate (and create) instances of the data-type
    it represents.
    """

    LAZY_TYPE_REGISTRY = {}  # more registrations
    HANDLER_REGISTRY = {}
    baseType = None
    typeConstant = None
    HANDLED_TYPES = ()
    preferredOutput = None
    isOutput = False
    GENERIC_OUTPUT_PREFERENCES = ['numpy', 'ctypesarrays']
    ALL_OUTPUT_HANDLERS = []

    def loadAll(cls):
        """Load all OpenGL.plugins-registered FormatHandler classes"""
        for entrypoint in plugins.FormatHandler.all():
            cls.loadPlugin(entrypoint)

    @classmethod
    def loadPlugin(cls, entrypoint):
        """Load a single entry-point via plugins module"""
        if not entrypoint.loaded:
            from OpenGL.arrays.arraydatatype import ArrayDatatype

            try:
                plugin_class = entrypoint.load()
            except ImportError as err:
                from OpenGL import logs
                from OpenGL._configflags import WARN_ON_FORMAT_UNAVAILABLE

                _log = logs.getLog('OpenGL.formathandler')
                if WARN_ON_FORMAT_UNAVAILABLE:
                    logFunc = _log.warn
                else:
                    logFunc = _log.info
                logFunc(
                    'Unable to load registered array format handler %s:\n%s',
                    entrypoint.name,
                    _log.getException(err),
                )
            else:
                handler = plugin_class()
                # handler.register( handler.HANDLED_TYPES )
                ArrayDatatype.getRegistry()[entrypoint.name] = handler
                return handler
            entrypoint.loaded = True

    @classmethod
    def typeLookup(cls, type):
        """Lookup handler by data-type"""
        from OpenGL.arrays.arraydatatype import ArrayDatatype

        registry = ArrayDatatype.getRegistry()
        try:
            return registry[type]
        except KeyError as err:
            key = '%s.%s' % (type.__module__, type.__name__)
            plugin = cls.LAZY_TYPE_REGISTRY.get(key)
            if plugin:
                cls.loadPlugin(plugin)
                return registry[type]
            raise KeyError("""Unable to find data-format handler for %s""" % (type,))

    loadAll = classmethod(loadAll)

    def register(self, types=None):
        """Register this class as handler for given set of types"""
        from OpenGL.arrays.arraydatatype import ArrayDatatype

        ArrayDatatype.getRegistry().register(self, types)

    def registerReturn(self):
        """Register this handler as the default return-type handler"""
        from OpenGL.arrays.arraydatatype import ArrayDatatype

        ArrayDatatype.getRegistry().registerReturn(self)

    def from_param(self, value, typeCode=None):
        """Convert to a ctypes pointer value"""

    def dataPointer(self, value):
        """return long for pointer value"""

    def asArray(self, value, typeCode=None):
        """Given a value, convert to array representation"""

    def arrayToGLType(self, value):
        """Given a value, guess OpenGL type of the corresponding pointer"""

    def arraySize(self, value, typeCode=None):
        """Given a data-value, calculate dimensions for the array"""

    def unitSize(self, value, typeCode=None):
        """Determine unit size of an array (if possible)"""
        if self.baseType is not None:
            return

    def dimensions(self, value, typeCode=None):
        """Determine dimensions of the passed array value (if possible)"""



'''OpenGL extension NV.pack_subimage

This module customises the behaviour of the 
OpenGL.raw.GLES2.NV.pack_subimage to provide a more 
Python-friendly API

Overview (from the spec)
	
	This OpenGL ES 2.0 extension adds support for GL_PACK_ROW_LENGTH_NV,
	GL_PACK_SKIP_ROWS_NV and GL_PACK_SKIP_PIXELS_NV as valid enums to
	PixelStore. The functionality is the same as in OpenGL. These are
	useful to update a sub-rectangle in host memory with data that can
	be read from the framebuffer or a texture (using FBO and texture
	attachments).

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/pack_subimage.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.NV.pack_subimage import *
from OpenGL.raw.GLES2.NV.pack_subimage import _EXTENSION_NAME

def glInitPackSubimageNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


"""Helper functions for wrapping array-using operations

These are functions intended to be used in wrapping
GL functions that deal with OpenGL array data-types.
"""

import OpenGL
import ctypes
from OpenGL import _configflags
from OpenGL import contextdata, error, converters
from OpenGL.arrays import arraydatatype
from OpenGL._bytes import bytes, unicode
import logging

_log = logging.getLogger('OpenGL.arrays.arrayhelpers')
from OpenGL import acceleratesupport

AsArrayTypedSizeChecked = None
if acceleratesupport.ACCELERATE_AVAILABLE:
    try:
        from OpenGL_accelerate.arraydatatype import AsArrayTypedSizeChecked
        from OpenGL_accelerate.wrapper import returnPyArgumentIndex
        from OpenGL_accelerate.arraydatatype import (
            AsArrayOfType,
            AsArrayTyped,
            AsArrayTypedSize,
        )
    except ImportError as err:
        _log.warning("Unable to load arrayhelpers accelerator from OpenGL_accelerate")
if AsArrayTypedSizeChecked is None:

    def returnPointer(
        result,
        baseOperation,
        pyArgs,
        cArgs,
    ):
        """Return the converted object as result of function

        Note: this is a hack that always returns pyArgs[0]!
        """
        return pyArgs[0]

    class AsArrayOfType(converters.PyConverter):
        """Given arrayName and typeName coerce arrayName to array of type typeName

        TODO: It should be possible to drop this if ERROR_ON_COPY,
        as array inputs always have to be the final objects in that
        case.
        """

        argNames = ('arrayName', 'typeName')
        indexLookups = (
            ('arrayIndex', 'arrayName', 'pyArgIndex'),
            ('typeIndex', 'typeName', 'pyArgIndex'),
        )

        def __init__(self, arrayName='pointer', typeName='type'):
            self.arrayName = arrayName
            self.typeName = typeName

        def __call__(self, arg, wrappedOperation, args):
            """Get the arg as an array of the appropriate type"""
            type = args[self.typeIndex]
            arrayType = arraydatatype.GL_CONSTANT_TO_ARRAY_TYPE[type]
            return arrayType.asArray(arg)

    class AsArrayTyped(converters.PyConverter):
        """Given arrayName and arrayType, convert arrayName to array of type

        TODO: It should be possible to drop this if ERROR_ON_COPY,
        as array inputs always have to be the final objects in that
        case.
        """

        argNames = ('arrayName', 'arrayType')
        indexLookups = (('arrayIndex', 'arrayName', 'pyArgIndex'),)

        def __init__(self, arrayName='pointer', arrayType=None):
            self.arrayName = arrayName
            self.arrayType = arrayType

        def __call__(self, arg, wrappedOperation, args):
            """Get the arg as an array of the appropriate type"""
            return self.arrayType.asArray(arg)

    class AsArrayTypedSize(converters.CConverter):
        """Given arrayName and arrayType, determine size of arrayName"""

        argNames = ('arrayName', 'arrayType')
        indexLookups = (('arrayIndex', 'arrayName', 'pyArgIndex'),)

        def __init__(self, arrayName='pointer', arrayType=None):
            self.arrayName = arrayName
            self.arrayType = arrayType

        def __call__(self, pyArgs, index, wrappedOperation):
            """Get the arg as an array of the appropriate type"""
            return self.arrayType.arraySize(pyArgs[self.arrayIndex])

else:
    returnPointer = returnPyArgumentIndex(0)

if not _configflags.ERROR_ON_COPY:

    def asArrayType(typ, size=None):
        """Create PyConverter to get first argument as array of type"""
        return converters.CallFuncPyConverter(typ.asArray)

else:

    def asArrayType(typ, size=None):
        """No converter required"""
        return None


if not _configflags.ARRAY_SIZE_CHECKING:
    asArrayTypeSize = asArrayType
else:
    if AsArrayTypedSizeChecked:
        asArrayTypeSize = AsArrayTypedSizeChecked
    else:

        def asArrayTypeSize(typ, size):
            """Create PyConverter function to get array as type and check size

            Produces a raw function, not a PyConverter instance
            """
            # asArray = typ.asArray
            dataType = typ.typeConstant
            # arraySize = typ.arraySize
            expectedBytes = ctypes.sizeof(typ.baseType) * size

            def asArraySize(incoming, function, args):
                handler = typ.getHandler(incoming)
                result = handler.asArray(incoming, dataType)
                # check that the number of bytes expected is present...
                byteSize = handler.arrayByteCount(result)
                if byteSize != expectedBytes:
                    raise ValueError(
                        """Expected %r byte array, got %r byte array"""
                        % (
                            expectedBytes,
                            byteSize,
                        ),
                        incoming,
                    )
                return result

            return asArraySize


if not _configflags.ERROR_ON_COPY:

    def asVoidArray():
        """Create PyConverter returning incoming as an array of any type"""
        from OpenGL.arrays import ArrayDatatype

        return converters.CallFuncPyConverter(ArrayDatatype.asArray)

else:

    def asVoidArray():
        """If there's no copying allowed, we can use default passing"""
        return None


class storePointerType(object):
    """Store named pointer value in context indexed by constant

    pointerName -- named pointer argument
    constant -- constant used to index in the context storage

    Note: OpenGL.STORE_POINTERS can be set with ERROR_ON_COPY
    to ignore this storage operation.

    Stores the pyArgs (i.e. result of pyConverters) for the named
    pointer argument...
    """

    def __init__(self, pointerName, constant):
        self.pointerName = pointerName
        self.constant = constant

    def finalise(self, wrapper):
        self.pointerIndex = wrapper.pyArgIndex(self.pointerName)

    def __call__(self, result, baseOperation, pyArgs, cArgs):
        contextdata.setValue(self.constant, pyArgs[self.pointerIndex])


def setInputArraySizeType(baseOperation, size, type, argName=0):
    """Decorate function with vector-handling code for a single argument

    if OpenGL.ERROR_ON_COPY is False, then we return the
    named argument, converting to the passed array type,
    optionally checking that the array matches size.

    if OpenGL.ERROR_ON_COPY is True, then we will dramatically
    simplify this function, only wrapping if size is True, i.e.
    only wrapping if we intend to do a size check on the array.
    """
    from OpenGL import wrapper

    return wrapper.wrapper(baseOperation).setInputArraySize(argName, size)


def arraySizeOfFirstType(typ, default):
    unitSize = typ.unitSize

    def arraySizeOfFirst(pyArgs, index, baseOperation):
        """Return the array size of the first argument"""
        array = pyArgs[0]
        if array is None:
            return default
        else:
            return unitSize(array)

    return arraySizeOfFirst



'''OpenGL extension EXT.shader_samples_identical

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.shader_samples_identical to provide a more 
Python-friendly API

Overview (from the spec)
	
	Multisampled antialiasing has become a common method for improving the
	quality of rendered images.  Multisampling differs from supersampling in
	that the color of a primitive that covers all or part of a pixel is
	resolved once, regardless of the number of samples covered.  If a large
	polygon is rendered, the colors of all samples in each interior pixel will
	be the same.  This suggests a simple compression scheme that can reduce
	the necessary memory bandwidth requirements.  In one such scheme, each
	sample is stored in a separate slice of the multisample surface.  An
	additional multisample control surface (MCS) contains a mapping from pixel
	samples to slices.
	
	If all the values stored in the MCS for a particular pixel are the same,
	then all the samples have the same value.  Applications can take advantage
	of this information to reduce the bandwidth of reading multisample
	textures.  A custom multisample resolve filter could optimize resolving
	pixels where every sample is identical by reading the color once.
	
	color = texelFetch(sampler, coordinate, 0);
	if (!textureSamplesIdenticalEXT(sampler, coordinate)) {
	    for (int i = 1; i < MAX_SAMPLES; i++) {
	        vec4 c = texelFetch(sampler, coordinate, i);
	
	        //... accumulate c into color
	
	    }
	}

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/shader_samples_identical.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.shader_samples_identical import *
from OpenGL.raw.GLES2.EXT.shader_samples_identical import _EXTENSION_NAME

def glInitShaderSamplesIdenticalEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension ARM.shader_core_properties

This module customises the behaviour of the 
OpenGL.raw.GLES2.ARM.shader_core_properties to provide a more 
Python-friendly API

Overview (from the spec)
	
	Applications may wish to know and/or control certain GPU shader core properties
	to select the optimal logic. This extension allows application to query
	and set the number of active GPU shader cores, and query other shader
	core properties for the optimal logic and performance profiling, including:
	
	  * The maximum number of physical shader cores available on the device;
	
	  * The number of active shader cores specified on the device;
	
	  * A bitfield indicating the presence of shader cores on the device;
	
	  * The maximum warp count per shader core available on the device;
	
	  * The maximum number of pixels output per clock per shader core;
	
	  * The maximum number of texels per clock per shader core;
	
	  * The maximum number of single-precision fused multiply-add operations per
	    clock per shader core;

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARM/shader_core_properties.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.ARM.shader_core_properties import *
from OpenGL.raw.GLES2.ARM.shader_core_properties import _EXTENSION_NAME

def glInitShaderCorePropertiesARM():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_EXT_texture_sRGB_decode'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_EXT_texture_sRGB_decode',error_checker=_errors._error_checker)
GL_DECODE_EXT=_C('GL_DECODE_EXT',0x8A49)
GL_SKIP_DECODE_EXT=_C('GL_SKIP_DECODE_EXT',0x8A4A)
GL_TEXTURE_SRGB_DECODE_EXT=_C('GL_TEXTURE_SRGB_DECODE_EXT',0x8A48)




'''OpenGL extension QCOM.ycbcr_degamma

This module customises the behaviour of the 
OpenGL.raw.GLES2.QCOM.ycbcr_degamma to provide a more 
Python-friendly API

Overview (from the spec)
	
	The OpenGL ES extensions OES_EGL_image_external and EXT_EGL_image_storage provide a mechanism for creating
	GL textures sharing storage with EGLImage objects which can encapsulate an external YCbCr buffer.
	The YCbCr data is stored according to the colorspace standards like ITU BT.601, BT.709, or BT.2020,
	and a transfer function like ITU OETF or sRGB EOTF, which translates the YCbCr data into non-linear space Y'CbCr.
	When sampling a Y'CbCr texture containing texels encoded, the conversion back to linear RGB requires
	conversion from non-linear RGB space to linear RGB space using an inverse transfer function.
	This operation of applying the inverse transfer function is also called "degamma".
	Currently, degamma operation is not executed as part of texture sampling, and instead
	application's shader code is expected to perform it separately. This computation can be somewhat expensive in the shader.
	
	This extension allows implementations to use "sRGB EOTF" inverse transform function defined in Khronos
	data format 1.3 specification, for degamma operation. The degamma is performed during texture filtering,
	allowing texture filtering to operate in a linear space. The extension provides this functionality by
	selectively enabling degamma for both the luminance{Y} and/or chrominance {CbCr} components of any
	8 bit YCbCr format.
	This new functionality is layered on top of the OES_EGL_image_external and EXT_EGL_image_storage
	extensions.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/QCOM/ycbcr_degamma.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.QCOM.ycbcr_degamma import *
from OpenGL.raw.GLES2.QCOM.ycbcr_degamma import _EXTENSION_NAME

def glInitYcbcrDegammaQCOM():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension VERSION.GL_4_5

This module customises the behaviour of the 
OpenGL.raw.GL.VERSION.GL_4_5 to provide a more 
Python-friendly API

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/VERSION/GL_4_5.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.VERSION.GL_4_5 import *
from OpenGL.raw.GL.VERSION.GL_4_5 import _EXTENSION_NAME

def glInitGl45VERSION():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glCreateTransformFeedbacks.ids size not checked against n
glCreateTransformFeedbacks=wrapper.wrapper(glCreateTransformFeedbacks).setInputArraySize(
    'ids', None
)
# INPUT glCreateBuffers.buffers size not checked against n
glCreateBuffers=wrapper.wrapper(glCreateBuffers).setInputArraySize(
    'buffers', None
)
# INPUT glNamedBufferStorage.data size not checked against size
glNamedBufferStorage=wrapper.wrapper(glNamedBufferStorage).setInputArraySize(
    'data', None
)
# INPUT glNamedBufferData.data size not checked against size
glNamedBufferData=wrapper.wrapper(glNamedBufferData).setInputArraySize(
    'data', None
)
# INPUT glNamedBufferSubData.data size not checked against size
glNamedBufferSubData=wrapper.wrapper(glNamedBufferSubData).setInputArraySize(
    'data', None
)
# INPUT glClearNamedBufferData.data size not checked against 'format,type'
glClearNamedBufferData=wrapper.wrapper(glClearNamedBufferData).setInputArraySize(
    'data', None
)
# INPUT glClearNamedBufferSubData.data size not checked against 'format,type'
glClearNamedBufferSubData=wrapper.wrapper(glClearNamedBufferSubData).setInputArraySize(
    'data', None
)
glGetNamedBufferPointerv=wrapper.wrapper(glGetNamedBufferPointerv).setInputArraySize(
    'params', 1
)
# INPUT glGetNamedBufferSubData.data size not checked against size
glGetNamedBufferSubData=wrapper.wrapper(glGetNamedBufferSubData).setInputArraySize(
    'data', None
)
# INPUT glCreateFramebuffers.framebuffers size not checked against n
glCreateFramebuffers=wrapper.wrapper(glCreateFramebuffers).setInputArraySize(
    'framebuffers', None
)
# INPUT glNamedFramebufferDrawBuffers.bufs size not checked against n
glNamedFramebufferDrawBuffers=wrapper.wrapper(glNamedFramebufferDrawBuffers).setInputArraySize(
    'bufs', None
)
# INPUT glInvalidateNamedFramebufferData.attachments size not checked against numAttachments
glInvalidateNamedFramebufferData=wrapper.wrapper(glInvalidateNamedFramebufferData).setInputArraySize(
    'attachments', None
)
# INPUT glInvalidateNamedFramebufferSubData.attachments size not checked against numAttachments
glInvalidateNamedFramebufferSubData=wrapper.wrapper(glInvalidateNamedFramebufferSubData).setInputArraySize(
    'attachments', None
)
# INPUT glClearNamedFramebufferiv.value size not checked against 'buffer'
glClearNamedFramebufferiv=wrapper.wrapper(glClearNamedFramebufferiv).setInputArraySize(
    'value', None
)
# INPUT glClearNamedFramebufferuiv.value size not checked against 'buffer'
glClearNamedFramebufferuiv=wrapper.wrapper(glClearNamedFramebufferuiv).setInputArraySize(
    'value', None
)
# INPUT glClearNamedFramebufferfv.value size not checked against 'buffer'
glClearNamedFramebufferfv=wrapper.wrapper(glClearNamedFramebufferfv).setInputArraySize(
    'value', None
)
# INPUT glCreateRenderbuffers.renderbuffers size not checked against n
glCreateRenderbuffers=wrapper.wrapper(glCreateRenderbuffers).setInputArraySize(
    'renderbuffers', None
)
# INPUT glCreateTextures.textures size not checked against n
glCreateTextures=wrapper.wrapper(glCreateTextures).setInputArraySize(
    'textures', None
)
# INPUT glCompressedTextureSubImage1D.data size not checked against imageSize
glCompressedTextureSubImage1D=wrapper.wrapper(glCompressedTextureSubImage1D).setInputArraySize(
    'data', None
)
# INPUT glCompressedTextureSubImage2D.data size not checked against imageSize
glCompressedTextureSubImage2D=wrapper.wrapper(glCompressedTextureSubImage2D).setInputArraySize(
    'data', None
)
# INPUT glCompressedTextureSubImage3D.data size not checked against imageSize
glCompressedTextureSubImage3D=wrapper.wrapper(glCompressedTextureSubImage3D).setInputArraySize(
    'data', None
)
# INPUT glTextureParameterfv.param size not checked against 'pname'
glTextureParameterfv=wrapper.wrapper(glTextureParameterfv).setInputArraySize(
    'param', None
)
# INPUT glTextureParameterIiv.params size not checked against 'pname'
glTextureParameterIiv=wrapper.wrapper(glTextureParameterIiv).setInputArraySize(
    'params', None
)
# INPUT glTextureParameterIuiv.params size not checked against 'pname'
glTextureParameterIuiv=wrapper.wrapper(glTextureParameterIuiv).setInputArraySize(
    'params', None
)
# INPUT glTextureParameteriv.param size not checked against 'pname'
glTextureParameteriv=wrapper.wrapper(glTextureParameteriv).setInputArraySize(
    'param', None
)
# INPUT glGetTextureImage.pixels size not checked against bufSize
glGetTextureImage=wrapper.wrapper(glGetTextureImage).setInputArraySize(
    'pixels', None
)
# INPUT glGetCompressedTextureImage.pixels size not checked against bufSize
glGetCompressedTextureImage=wrapper.wrapper(glGetCompressedTextureImage).setInputArraySize(
    'pixels', None
)
# INPUT glCreateVertexArrays.arrays size not checked against n
glCreateVertexArrays=wrapper.wrapper(glCreateVertexArrays).setInputArraySize(
    'arrays', None
)
# INPUT glVertexArrayVertexBuffers.buffers size not checked against count
# INPUT glVertexArrayVertexBuffers.offsets size not checked against count
# INPUT glVertexArrayVertexBuffers.strides size not checked against count
glVertexArrayVertexBuffers=wrapper.wrapper(glVertexArrayVertexBuffers).setInputArraySize(
    'buffers', None
).setInputArraySize(
    'offsets', None
).setInputArraySize(
    'strides', None
)
# INPUT glCreateSamplers.samplers size not checked against n
glCreateSamplers=wrapper.wrapper(glCreateSamplers).setInputArraySize(
    'samplers', None
)
# INPUT glCreateProgramPipelines.pipelines size not checked against n
glCreateProgramPipelines=wrapper.wrapper(glCreateProgramPipelines).setInputArraySize(
    'pipelines', None
)
# INPUT glCreateQueries.ids size not checked against n
glCreateQueries=wrapper.wrapper(glCreateQueries).setInputArraySize(
    'ids', None
)
# INPUT glGetTextureSubImage.pixels size not checked against bufSize
glGetTextureSubImage=wrapper.wrapper(glGetTextureSubImage).setInputArraySize(
    'pixels', None
)
# INPUT glGetCompressedTextureSubImage.pixels size not checked against bufSize
glGetCompressedTextureSubImage=wrapper.wrapper(glGetCompressedTextureSubImage).setInputArraySize(
    'pixels', None
)
# INPUT glGetnCompressedTexImage.pixels size not checked against bufSize
glGetnCompressedTexImage=wrapper.wrapper(glGetnCompressedTexImage).setInputArraySize(
    'pixels', None
)
# INPUT glGetnTexImage.pixels size not checked against bufSize
glGetnTexImage=wrapper.wrapper(glGetnTexImage).setInputArraySize(
    'pixels', None
)
# INPUT glReadnPixels.data size not checked against bufSize
glReadnPixels=wrapper.wrapper(glReadnPixels).setInputArraySize(
    'data', None
)
# INPUT glGetnMapdv.v size not checked against 'bufSize'
glGetnMapdv=wrapper.wrapper(glGetnMapdv).setInputArraySize(
    'v', None
)
# INPUT glGetnPixelMapfv.values size not checked against 'bufSize'
glGetnPixelMapfv=wrapper.wrapper(glGetnPixelMapfv).setInputArraySize(
    'values', None
)
# INPUT glGetnPolygonStipple.pattern size not checked against bufSize
glGetnPolygonStipple=wrapper.wrapper(glGetnPolygonStipple).setInputArraySize(
    'pattern', None
)
# INPUT glGetnColorTable.table size not checked against bufSize
glGetnColorTable=wrapper.wrapper(glGetnColorTable).setInputArraySize(
    'table', None
)
# INPUT glGetnConvolutionFilter.image size not checked against bufSize
glGetnConvolutionFilter=wrapper.wrapper(glGetnConvolutionFilter).setInputArraySize(
    'image', None
)
# INPUT glGetnSeparableFilter.column size not checked against columnBufSize
# INPUT glGetnSeparableFilter.row size not checked against rowBufSize
glGetnSeparableFilter=wrapper.wrapper(glGetnSeparableFilter).setInputArraySize(
    'column', None
).setInputArraySize(
    'row', None
).setInputArraySize(
    'span', 0
)
# INPUT glGetnHistogram.values size not checked against bufSize
glGetnHistogram=wrapper.wrapper(glGetnHistogram).setInputArraySize(
    'values', None
)
# INPUT glGetnMinmax.values size not checked against bufSize
glGetnMinmax=wrapper.wrapper(glGetnMinmax).setInputArraySize(
    'values', None
)
### END AUTOGENERATED SECTION



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_SGIX_texture_lod_bias'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_SGIX_texture_lod_bias',error_checker=_errors._error_checker)
GL_TEXTURE_LOD_BIAS_R_SGIX=_C('GL_TEXTURE_LOD_BIAS_R_SGIX',0x8190)
GL_TEXTURE_LOD_BIAS_S_SGIX=_C('GL_TEXTURE_LOD_BIAS_S_SGIX',0x818E)
GL_TEXTURE_LOD_BIAS_T_SGIX=_C('GL_TEXTURE_LOD_BIAS_T_SGIX',0x818F)




'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES1 import _types as _cs
# End users want this...
from OpenGL.raw.GLES1._types import *
from OpenGL.raw.GLES1 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES1_APPLE_framebuffer_multisample'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES1,'GLES1_APPLE_framebuffer_multisample',error_checker=_errors._error_checker)
GL_DRAW_FRAMEBUFFER_APPLE=_C('GL_DRAW_FRAMEBUFFER_APPLE',0x8CA9)
GL_DRAW_FRAMEBUFFER_BINDING_APPLE=_C('GL_DRAW_FRAMEBUFFER_BINDING_APPLE',0x8CA6)
GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE_APPLE=_C('GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE_APPLE',0x8D56)
GL_MAX_SAMPLES_APPLE=_C('GL_MAX_SAMPLES_APPLE',0x8D57)
GL_READ_FRAMEBUFFER_APPLE=_C('GL_READ_FRAMEBUFFER_APPLE',0x8CA8)
GL_READ_FRAMEBUFFER_BINDING_APPLE=_C('GL_READ_FRAMEBUFFER_BINDING_APPLE',0x8CAA)
GL_RENDERBUFFER_SAMPLES_APPLE=_C('GL_RENDERBUFFER_SAMPLES_APPLE',0x8CAB)
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,_cs.GLsizei)
def glRenderbufferStorageMultisampleAPPLE(target,samples,internalformat,width,height):pass
@_f
@_p.types(None,)
def glResolveMultisampleFramebufferAPPLE():pass
